/* acl.c File security modifyers

   Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "ftpd.h"

/* This adds the ACL characters to an ACL string. Characters that are already
   in the ACL string are not duplicated. The ACL chars are expanded if they
   state ALL or NONE. Note that the ACL string won't be added to if it
   contains an X */

void addaccess(char *outdata, char *accs)
{
	char acc[128];
	int c;
	
	if (strchr(outdata, 'X') != NULL)
		return;
		
	strcpy(acc, accs);
	if (strlen(acc) > 0)
		if (acc[strlen(acc) - 1] == 10)
			acc[strlen(acc) - 1] = 0;

	if (strcmp(acc, "ALL") == 0) 
		strcpy(acc, "DALRW");
	if (strcmp(acc, "NONE") == 0)
		strclear(acc);
	
	for( c = 0; c < strlen(acc); c++ )
	{
		switch(acc[c])
		{
			case 'X':
			case 'A':
			case 'D':
			case 'L':
			case 'R':
			case 'W': if (strchr(outdata, acc[c]) == NULL)
				  {
					int a = strlen(outdata);
			
					outdata[a] = acc[c];
					outdata[a+1] = 0;
				  }
		}
	}
}

void addaclstr(FTPSTATE *peer, char *accstr)
{
	int count = 0;
	int result = 0;
	char *accs; 
	char *dir;

	accs = strchr(accstr, ':');
	if (accs)
	{		 
		*accs = 0;
		accs++;    /* break the string into two at the : */
		
		dir = strdupwrapper(accstr);
		
		while(!result)
		{
			if (peer->acldata[count].pathlen == 0)
				result = 1;
			else if (strcmp(peer->acldata[count].path, dir) == 0)			
				result = 2;
			if (!result)
				count++;
		}
	
		if (result == 1)
		{
			reallocwrapper(sizeof(ACLCACHEENT) * (count + 2), (void *)&(peer->acldata));
			peer->acldata[count].path = dir;
			peer->acldata[count].perms = mallocwrapper(10);
			peer->acldata[count].pathlen = strlen(dir);
			strclear(peer->acldata[count].perms);
			addaccess(peer->acldata[count].perms, accs);
			count++;
			peer->acldata[count].pathlen = 0;
		}
		else
		{
			addaccess(peer->acldata[count].perms, accs);
		}
	}
}		


void acl_clearcache(FTPSTATE *peer)
{
#if DEBUG
	printf("[acl_clearcache]\n");
#endif
	if (peer->acldata != NULL)
	{
		int count = 0;
		while(peer->acldata[count].pathlen != 0)
		{
			freewrapper(peer->acldata[count].perms);
			freewrapper(peer->acldata[count].path);
			count++;
		}
		freewrapper(peer->acldata);
		peer->acldata = NULL;
	}
}

void acl_makecache(FTPSTATE *peer)
{
#if DEBUG
	printf("[acl_makecache]\n");
#endif
	if (peer->acldata == NULL)
	{
		peer->acldata = mallocwrapper(sizeof(ACLCACHEENT));
		peer->acldata[0].pathlen = 0; 
	}
	else
	{
		errormsg("acl_makecache failure: tried to overwrite access list", __FILE__, __LINE__);
	}
}					

char *acl_getinfo(FTPSTATE *peer, char *pathname)
{
	int dirlen;
	char *dir = strdupwrapper(pathname);
	char *result = NULL;

	assert(peer->acldata != NULL);
	
	dir_tovirtual(peer, dir);
		
	do
	{
		int count = 0;
		dirlen = strlen(dir);
		while((peer->acldata[count].pathlen != 0) && (result == NULL))
		{
			if (peer->acldata[count].pathlen == dirlen)
				if (strcmp(peer->acldata[count].path, dir) == 0)
					result = peer->acldata[count].perms;
			count++;
		}

		if ((result == NULL) && (dirlen > 1))
		{
			char *last_slash;
			if (dir[dirlen - 1] == '/')
				dir[dirlen - 1] = 0;
			last_slash = strrchr(dir, '/');
			last_slash++;
			*last_slash = 0;
		}
							
	} while ((result == NULL) && (dirlen > 1));
			
	freewrapper(dir);
	return(result);
}

int check_acl(FTPSTATE *peer, char *pathname, char perm)
{
	char *data = acl_getinfo(peer, pathname);
	int result = FALSE;
	
	if (data != NULL)
		result = (strchr(data, perm) != NULL);
	
	if (result)
	{
		errno = 0;
		return(0);
	}
	
	errno = EACCES;
	return(-1);
}
