/* Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "ftpd.h"
#include "auth.h"

/* This file contains code to autheticate against unix password files */

typedef struct 
{
	char *udata;
	char *password;
	char *username;
	char *homedir;
	char *rootdir;
} PASSWDSTRUCT;

int internalauth_config(char *username, char *data, int line, void *h)
{
	PASSWDSTRUCT *handle = (PASSWDSTRUCT *)h;
	
	if (strcmp(username, handle->username) == 0)
	{
		handle->udata = strdupwrapper(data);
		return(FALSE);
	}
	return(TRUE);
}

void *internalauth_gethandle(char *username, char *params)
{
	PASSWDSTRUCT *newhandle;
	int result;
	
	newhandle = mallocwrapper(sizeof(PASSWDSTRUCT));
	newhandle->username = strdupwrapper(username);

	if (params == NULL)
		goto logerror;
		
	result = loadconfigfile(params, internalauth_config, newhandle);
	
	switch (result)
	{
		case FALSE: { goto logerror; };
		case TRUE: { goto error; };
	}

	if (strchrcount(newhandle->udata, ':') != 2)
	{
		freewrapper(newhandle->udata);
		goto error;
	}
	
	newhandle->password = newhandle->udata;
	newhandle->homedir = strchr(newhandle->udata, ':') + 1;
	/* lets hope this doesn't cause heart attacks */
	newhandle->homedir[-1] = 0;
	newhandle->rootdir = strchr(newhandle->homedir, ':') + 1;
	newhandle->rootdir[-1] = 0;
	
	return(newhandle);

logerror:
	if (params == NULL)
		log_addentry(MYLOG_INFO, 0, NULL, "You didn't specify a password file for internal authentication!");
	else
	{
		char *msg = safe_snprintf("Internal password file '%s' doesn't exist", params);
		log_addentry(MYLOG_INFO, 0, NULL, msg);
		freewrapper(msg);
	}
error:
	freewrapper(newhandle->username);
	freewrapper(newhandle);

	return(NULL);
}

void internalauth_freehandle(void *h)
{
	PASSWDSTRUCT *handle = (PASSWDSTRUCT *)h;
	
	freewrapper(handle->udata);
	freewrapper(handle->username);
	freewrapper(handle);
}

int internalauth_checkpasswd(void *h, char *password, char **errmsg)
{
	PASSWDSTRUCT *handle = (PASSWDSTRUCT *)h;
	return(chkpassword(handle->password, password));
}

char *internalauth_gethomedir(void *h)
{
	return(((PASSWDSTRUCT *)h)->homedir);
}

char *internalauth_getrootdir(void *h)
{
	return(((PASSWDSTRUCT *)h)->rootdir);
}

PERMSTRUCT internalauth_commands =
{
	internalauth_checkpasswd,
	internalauth_gethomedir,
	internalauth_getrootdir,
	NULL,
	NULL,
	internalauth_gethandle,
	internalauth_freehandle,
};
