/* Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifdef MUDAUTH

#include "ftpd.h"
#include "auth.h"

/* This file contains code to autheticate for mud users */

typedef struct 
{
	char *username;
	char *password;
	int level;
} PASSWDSTRUCT;

int mudauth_config(char *cmd, char *data, int line, void *h)
{
	PASSWDSTRUCT *handle = (PASSWDSTRUCT *)h;
	
	if (strcmp(cmd, "Level") == 0)
		sscanf(data, "%d", &(handle->level));
	else if (strcmp(cmd, "Password") == 0)
	{
		handle->password = strdupwrapper(data + 1);
		handle->password[13] = 0;
	}
}
				
void *mudauth_gethandle(char *username, char *params)
{
	PASSWDSTRUCT *newhandle;
	int result;

	if (params == NULL)
		return(NULL);

	newhandle = mallocwrapper(sizeof(PASSWDSTRUCT));
	newhandle->username = strdupwrapper(username);
	newhandle->password = NULL;
	newhandle->level = 0;
	
	result = loadconfigfile(params, mudauth_config, newhandle);
	
	if (newhandle->level <= 20)
	{
		freewrapper(newhandle->username);
		if (newhandle->password) freewrapper(newhandle->password);
		newhandle = NULL;
	}
	return(newhandle);
}

void mudauth_freehandle(void *h)
{
	PASSWDSTRUCT *handle = (PASSWDSTRUCT *)h;
	
	freewrapper(handle->password);
	freewrapper(handle->username);
	freewrapper(handle);
}

int mudauth_checkpasswd(void *h, char *password, char **errmsg)
{
	PASSWDSTRUCT *handle = (PASSWDSTRUCT *)h;
	
	return(chkpassword(handle->password, password));
}

PERMSTRUCT mudauth_commands =
{
	mudauth_checkpasswd,
	NULL,
	NULL,
	NULL,
	NULL,
	mudauth_gethandle,
	mudauth_freehandle,
};

#endif
