/* Copyright (C) 1999 Beau Kuiper
   this is a quick hack for mudpasswd, servicable 1 nighter
   warning, this code is not buffer overflow safe. DONT RUN AS SETUID ROOT

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "ftpd.h"
#include "auth.h"

#ifdef HAVE_PAM_START

#include <security/pam_appl.h>

/* This file contains code to autheticate using PAM password. It was written
   using proftpd as a reference */

typedef struct 
{
	char *password;
	char *username;
	struct passwd *passent;
	pam_handle_t *pamh;
	int retval;
} PASSWDSTRUCT;
 
PASSWDSTRUCT *currenthandle = NULL;

static int pamauth_conv(int num_msg, struct pam_message **msg,
			struct pam_response **resp,
			void *appdata_ptr)
{
	struct pam_response *response = NULL;
 	int count;

 	/* I cannot use mallocwrapper here because it will break counters
 	   since PAM takes the memory */
 	response = malloc(sizeof(struct pam_response) * num_msg);
 	
 	if (response == NULL)
 		return(PAM_CONV_ERR);
 	
 	for (count = 0; count < num_msg; count++)
 	{
 		response[count].resp_retcode = PAM_SUCCESS;
 		switch(msg[count]->msg_style)
 		{
 			case PAM_PROMPT_ECHO_ON:
 				if (currenthandle == NULL)
 					ERRORMSGFATAL("PAM is being mean! It wants a username I don't know");
 				response[count].resp = strdup(currenthandle->username);
 				if (!response[count].resp)
 					return(PAM_CONV_ERR);
 				break;
 			case PAM_PROMPT_ECHO_OFF:
				if (currenthandle == NULL)
 					ERRORMSGFATAL("PAM is being mean! It wants a password I don't know");
  				response[count].resp = currenthandle->password;
 				break; 
 			case PAM_TEXT_INFO: 
 			case PAM_ERROR_MSG:
 				response[count].resp = NULL;
 				break;
 			
 			default:	/* yuck, an error! */
 				free(response);	
				return(PAM_CONV_ERR);
		}
	}
	
	*resp = response;
	return(PAM_SUCCESS);
}	

static struct pam_conv mypam = {
  &pamauth_conv,
  NULL
};

void pamauth_freehandle(void *h)
{
	PASSWDSTRUCT *handle = (PASSWDSTRUCT *)h;
	pam_end(handle->pamh, handle->retval);

	freewrapper(handle->username);
	freewrapper(handle);
}
 	
void *pamauth_gethandle(char *username, char *params)
{
	PASSWDSTRUCT *newhandle;
	int result;
	
	newhandle = mallocwrapper(sizeof(PASSWDSTRUCT));
	newhandle->username = strdupwrapper(username);

	if (params == NULL)
		params = "ftp";
	
	newhandle->retval = pam_start(params, username, &mypam, &(newhandle->pamh));
	if (newhandle->retval != PAM_SUCCESS)
		goto error;
	
	newhandle->passent = getpwnam(username);
	if (newhandle->passent == NULL)
		goto error;
	
	return(newhandle);

error:
	pamauth_freehandle(newhandle);
	return(NULL);
}

int pamauth_checkpasswd(void *h, char *password, char **errmsg)
{
	PASSWDSTRUCT *handle = (PASSWDSTRUCT *)h;
	
	currenthandle = handle;
	handle->password = strdup(password);
	if (!handle->password)
		ERRORMSGFATAL("out of memory in PAM module!");
	
	handle->retval = pam_authenticate(handle->pamh, PAM_SILENT);
	
	if (handle->retval == PAM_SUCCESS)
		handle->retval = pam_acct_mgmt(handle->pamh, 0);
	else
		return(FALSE);
	
	if (handle->retval == PAM_SUCCESS)
		handle->retval = pam_setcred(handle->pamh, PAM_ESTABLISH_CRED);
	else
	{
		*errmsg = safe_snprintf("Account Problem");
		return(FALSE);
	}
	
	if (handle->retval == PAM_SUCCESS)
		return(TRUE);

	*errmsg = safe_snprintf("Credantial Problem");
	return(FALSE);
}

char *pamauth_gethomedir(void *h)
{
	return(((PASSWDSTRUCT *)h)->passent->pw_dir);
}

char *pamauth_getrootdir(void *h)
{
	return("/");
}

uid_t pamauth_getuseruid(void *h)
{
	return(((PASSWDSTRUCT *)h)->passent->pw_uid);
}

gid_t pamauth_getusergid(void *h)
{
	return(((PASSWDSTRUCT *)h)->passent->pw_gid);
}

PERMSTRUCT pamauth_commands =
{
	pamauth_checkpasswd,
	pamauth_gethomedir,
	pamauth_getrootdir,
	pamauth_getuseruid,
	pamauth_getusergid,
	pamauth_gethandle,
	pamauth_freehandle,
};

#endif
