/* Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "ftpd.h"

#define OUTBUFFTRIGGER 		2048
#define OUTBUFFMAX		4096

/* this will write data if fd is valid, else pretend data is written */

int write2(int fd, void *dat, int leng)
{
	if (fd >= 0)
		return(write(fd, dat, leng));
	else
		return(leng);
}
		
OUTBUFF *obuff_new(int outsock)
{
	OUTBUFF *new = mallocwrapper(sizeof(OUTBUFF));
	
	new->size = 0;
	new->fd = outsock;
	new->outblock = mallocwrapper(OUTBUFFMAX);
	return(new);
}

int obuff_writes(OUTBUFF *bout, char *output)
{
	return(obuff_write(bout, (void *)output, strlen(output)));
}

int obuff_write(OUTBUFF *bout, void *output, int outlen)
{
	if ((outlen + bout->size) > OUTBUFFMAX) 
		reallocwrapper(outlen + bout->size + 1, (void *)&(bout->outblock));
	
	memcpy(&(bout->outblock[bout->size]), output, outlen);	
	bout->size += outlen;
	
	/* flush as much data as we need */
	while (bout->size >= OUTBUFFTRIGGER)
	{
		int out = write2(bout->fd, bout->outblock, OUTBUFFTRIGGER);
		
		if (out > 0)
		{
			bout->size -= out;
			memmove(bout->outblock, &(bout->outblock[out]), bout->size);
		}
		else
		{
			bout->fd = -1;
			return(-1);
		}
	}
	if (bout->fd >= 0)
		return(outlen);
	else
		return(-1);
}

int obuff_flush(OUTBUFF *bout)
{
	while (bout->size > 0)
	{
		int out = MINIMUM(bout->size, OUTBUFFTRIGGER);
		
		out = write(bout->fd, bout->outblock, out);
		
		if (out > 0)
		{
			bout->size -= out;
			memmove(bout->outblock, &(bout->outblock[out]), bout->size);
		}
		else
		{
			bout->fd = -1;
			return(-1);
		}
	}
	if (bout->fd >= 0)
		return(0);
	else
		return(-1);
	
}

void obuff_free(OUTBUFF *bout)
{
	freewrapper(bout->outblock);
	freewrapper(bout);
}
