/* config.c --- gereric config file loader
   Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "ftpd.h"

/* this, rather messyly, splits an input line into the command and data
   where the command is the first word on a line, and the data is the
   second.
*/

char *strsplit(char *in, char *cmd, char *data)
{
	int pos = 0;
	int pos2 = 0;
	
	if (in[strlen(in) - 1] == 10)
		in[strlen(in) - 1] = 0;
		
	while((in[pos] <= 32) && (in[pos] != 0))
		pos++;
	if ((in[pos] == '#') || (in[pos] == 0))
		return(NULL);
	
	while((in[pos] != 32) && (in[pos] != 0))
	{
		cmd[pos2] = in[pos];
		pos2++;
		pos++;
	}
	
	if (in[pos] == 0)
		return(NULL);
	
	cmd[pos2] = 0;
	
	while((in[pos] <= 32) && (in[pos] != 0))
		pos++;
	if (in[pos] == 0)
		return(NULL);

	pos2 = 0;
	while((in[pos] != 32) && (in[pos] != 0))
	{
		if ((in[pos] == '\\') && (in[pos+1] != 0))
			pos++;
		
		data[pos2] = in[pos];
		pos2++;
		pos++;
	}
	
	data[pos2] = 0;
	return(data);
}

/* This will open and load each line in the specified config file (filename)
   and then split it up into constitute parts before sending them off to
   a handler you specify with a void pointer (to be used as the structure
   that the config file will load each item into) */

int loadconfigfile(char *filename, int (* confighandler)(char *, char *, int, void *), void *configdata)
{
	NEWFILE *configfile;
	char *inputline;
	char *cmd = NULL;
	char *data = NULL;
	int result = TRUE;
	int linenum = 0;
	
	configfile = nfopen(filename);
	if (configfile == NULL)
		return(FALSE);
		
	while((inputline = nfgetcs(configfile, '\n')) != NULL)
	{
		int a = strlen(inputline) + 1;
		reallocwrapper(a, (void *)&cmd);
		reallocwrapper(a, (void *)&data);
		linenum++;
		
		/* This is a fix for config files written with DOS
		   based software */
		 
		a = strlen(inputline);
 		if (inputline[a-2] == '\r')
			inputline[a-2] = 0;
		
		if (strsplit(inputline, cmd, data) != NULL)
			if (!confighandler(cmd, data, linenum, configdata))
			{
				result = 2;
				freewrapper(inputline);
				break;
			}
		freewrapper(inputline);
	}
	
	freewrapper(cmd);
	freewrapper(data);
	nfclose(configfile);
	return(result);
}
