/* Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "ftpd.h"

int nullfilter(char *data, int *len)
{
	return(*len);
}

int datasock_connect(SELECTER *sel, void *peerv)
{
	FTPSTATE *peer = (FTPSTATE *)peerv;
	DATAPORT *dport = peer->dport;
	int error = 0;
	int ret = FALSE;
	
	if (dport->passive)
	{
		unsigned int ip;
		int temp = get_conn(dport->passive, &ip);
		
		if (ip != peer->remoteip)
		{
			ftp_write(peer, 550, "Passive connection not from your Host!");
			close(temp);
			if (dport->filefd != -1)
				select_delfd(sel, dport->filefd);
			closedatasocket(peer);
			return(2);
		}
		else
		{
			select_takeread(sel, dport->passive);
			dport->remotefd = temp;
			select_addfd(sel, temp);
		}
		ret = 2; /* delete the listening socket */
	}
	else
	{
		int intsize = sizeof(int);
		getsockopt(dport->remotefd, SOL_SOCKET, SO_ERROR, &error, &intsize);
		select_takewrite(sel, dport->remotefd);
	}
	
	if (error != 0)
	{
		ftp_write(peer, 550, "Socket error connecting data port: %s", strerror(error));
		if (dport->filefd != -1)
			select_delfd(sel, dport->filefd);
		closedatasocket(peer);
		ret = 2;
	}
	else
	{
		/* Now attempt to throttle data port to maximum throughput
		   and NOPUSH */
		int on = LOWATERMARK;
	/* I disabled logging of setsockopt failure becuase it isn't fatal
	   and seemed to annoy people too much */
#ifdef SO_SNDLOWAT		/* If we can set it */
		setsockopt(dport->remotefd, SOL_SOCKET, SO_SNDLOWAT, &on, sizeof(int));
		on = LOWATERMARK;
#endif
#ifdef SO_RCVLOWAT
		setsockopt(dport->remotefd, SOL_SOCKET, SO_RCVLOWAT, &on, sizeof(int));
#endif
		if (dport->filter == nullfilter)
		{
			if (dport->tsize != -1)
 	 			ftp_write(peer, 150, "Starting BINARY transfer at offset %d (%d bytes)", dport->startpos, dport->tsize + dport->startpos);
			else
				ftp_write(peer, 150, "Starting BINARY transfer at offset %d", dport->startpos);
		}
		else
			ftp_write(peer, 150, "Starting ASCII transfer");
		dport->graceful = TRUE;
		if (dport->filefd == -1) /* if we don't have a file descriptor to read from */
			select_addwrite(sel, dport->remotefd, dport->firstop, peer); 
		else
	 		select_addread(sel, *(dport->fdin), dport->firstop, peer);
       	}	
       	return(ret);
} 

int startdatasocket(FTPSTATE *peer, int filefd, int (* asciifilter)(char *, int *))
{
	DATAPORT *newdport = mallocwrapper(sizeof(DATAPORT));
	newdport->filefd = filefd;
	newdport->pos = 0;
	newdport->startpos = peer->restartpos;
	peer->restartpos = 0;
	newdport->buffersize = 0;
	newdport->buffer = mallocwrapper(BUFFERSIZE * 2);
	newdport->passive = peer->passiveport;
	newdport->die = FALSE;
	newdport->tsize = 0;
	newdport->graceful = FALSE;
	if (peer->binary)
		newdport->filter = nullfilter;
	else
		newdport->filter = asciifilter;
	
	if (peer->passiveport == 0)
		newdport->remotefd = conn_server_nonblocking(peer->remoteip, peer->remoteport, peer->connport-1);
	else
		newdport->remotefd = peer->passiveport;

	peer->remoteport = peer->connport - 1;
	peer->passiveport = FALSE;
	peer->dport = newdport;
	
	if (newdport->remotefd == -1)
	{	
		int olderrno = errno; /* save errno for later */
		if (filefd != -1)
			close(filefd);
		errno = olderrno;
		closedatasocket(peer);
		return(-1);
	}
	else 
	{
		if (!newdport->passive)
			select_addfd(peer->sel, newdport->remotefd);
		if (filefd != -1)
			select_addfd(peer->sel, filefd);
		if (peer->dport->passive)
			select_addread(peer->sel, newdport->remotefd, datasock_connect, peer);
		else
			select_addwrite(peer->sel, newdport->remotefd, datasock_connect, peer);
	}
	
	return(0);
}	
	
void closedatasocket(FTPSTATE *peer)
{
	freewrapper(peer->dport->buffer);
	freewrapper(peer->dport);
	peer->dport = NULL;
	pnums_setop("idle", peer->threadnum);
}

void abortdatasocket(FTPSTATE *peer)
{
	if (peer->dport)
	{
		if (peer->dport->graceful)
		{
			select_delfd(peer->sel, peer->dport->remotefd);
			if (peer->dport->filefd != -1)
				select_delfd(peer->sel, peer->dport->filefd);
			ftp_write(peer, 426, "Transfer aborted. %d bytes transferred", MAXIMUM(0, peer->dport->pos - peer->dport->startpos));
		}
		else
		{
			select_delfd(peer->sel, peer->dport->remotefd);
			if (peer->dport->filefd != -1)
				select_delfd(peer->sel, peer->dport->filefd);
			ftp_write(peer, 530, "Data connection in progress cancelled");
		}	
		closedatasocket(peer);
	}
}
