/* Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "ftpd.h"
#include "ftpcmd.h"

extern FTPCMD mainftpcmd[];

/* This will print a NICE! error to the user */

void reporterror(FTPSTATE *peer, char *filename, int errorno)
{
	ftp_write(peer, 550, "'%s': %s", filename, strerror(errorno));
}

int readipstr(char *data, int *port)
{
	int a1,a2,a3,a4,a5,a6, result;
	
	result = sscanf(data, "%d,%d,%d,%d,%d,%d",&a1, &a2, &a3, &a4, &a5, &a6);
	if (result != 6)
		return(FALSE);
	
	*port = (a5 * 256) + a6;
	return(TRUE);
}

int ftp_dele(FTPSTATE *peer, char *filename)
{
	if (file_unlink(peer, filename) == 0)
		ftp_write(peer, 250, "Deleting file '%s'", filename);
	else
		reporterror(peer, filename, errno); 

	return(FALSE);
}

int ftp_chmod(FTPSTATE *peer, char *filename, int mode)
{
	if (file_chmod(peer, filename, mode) == 0)
		ftp_write(peer, 250, "Changing mode on file %s to %03o.", filename, mode); 
	else
		reporterror(peer, filename, errno); 

	return(FALSE);
}

int ftp_size(FTPSTATE *peer, char *filename)
{
	int filefd;
	int size, asize;
	struct stat statdata;
	
	if (peer->binary)
	{
		filefd = file_stat(peer, filename, &statdata);
	 	if (filefd == 0)
	 		size = statdata.st_size;
	}
	else
	{
		size = 0;
		filefd = file_open(peer, filename, O_RDONLY);
		if (filefd > 0)
		{
			char *chin, buffer[BUFFERSIZE];
				
			asize = read(filefd, buffer, BUFFERSIZE-1);
			while (asize > 0)
			{
				buffer[asize] = 0;
				size += asize;
				chin = buffer;
				while ((chin = strchr(chin, 10)) != NULL)
				{
					chin++;
					size++;
				}
				asize = read(filefd, buffer, BUFFERSIZE-1);
			}
			close(filefd);
		}
	}

	if (filefd >= 0)
		ftp_write(peer, 213, "%d", size); 
	else
		reporterror(peer, filename, errno);

 	return(FALSE);
}

int ftp_mdtm(FTPSTATE *peer, char *filename)
{
	struct stat statdata;
	
	if (file_stat(peer, filename, &statdata) == 0)
	{
		char timestr[20];
		struct tm *resulttm = gmtime(&statdata.st_mtime);
		
		strftime(timestr, 20, "%Y%m%d%H%M%S", resulttm);

		ftp_write(peer, 213, timestr); 
	}
	else
		reporterror(peer, filename, errno); 

	return(FALSE);
}

void setdumptokens(FTPSTATE *peer, TOKENSET *ts)
{
	char *outstr;
	unsigned int ip = peer->remoteip;
	time_t mytime = time(NULL);
		
	outstr = ctime(&mytime);
	outstr[strlen(outstr) - 1] = 0;
	tokenset_settoken(ts, 'T', strdupwrapper(outstr));
	tokenset_settoken(ts, 'U', strdupwrapper(peer->username));
	if (peer->pwd)
	{
		dir_tovirtual(peer, peer->pwd);
		tokenset_settoken(ts, 'C', strdupwrapper(peer->pwd));
		dir_toreal(peer, peer->pwd);
	}
	tokenset_settoken(ts, 'E', strdupwrapper(config->email));
	tokenset_settoken(ts, 'M', safe_snprintf("%d", config->maxthreads));
	tokenset_settoken(ts, 'N', safe_snprintf("%d", pnums_numusers()));
	tokenset_settoken(ts, 'R', safe_snprintf("%u.%u.%u.%u",
				(unsigned char)(ip >> 24),
				(unsigned char)((ip >> 16) & 255),
				(unsigned char)((ip >> 8) & 255),
				(unsigned char)((ip) & 255)));
	tokenset_settoken(ts, 'L', gethostname2());
	tokenset_settoken(ts, 'f', safe_snprintf("%d", peer->downloadedfiles));
	tokenset_settoken(ts, 'F', safe_snprintf("%d", peer->uploadedfiles));
	tokenset_settoken(ts, 'b', safe_snprintf("%d", peer->downloadedfilebytes));
	tokenset_settoken(ts, 'B', safe_snprintf("%d", peer->uploadedfilebytes));
	tokenset_settoken(ts, 'I', safe_snprintf("%d", peer->listdownloadedbytes));
	tokenset_settoken(ts, 'i', safe_snprintf("%d", peer->listconns));
	tokenset_settoken(ts, 'D', safe_snprintf("%d", peer->listdownloadedbytes + peer->downloadedfilebytes));
	tokenset_settoken(ts, 't', safe_snprintf("%d", peer->listdownloadedbytes + peer->downloadedfilebytes + peer->uploadedfilebytes));
	tokenset_settoken(ts, 'c', safe_snprintf("%d", peer->listconns + peer->downloadedfiles + peer->uploadedfiles));
	tokenset_settoken(ts, 'd', safe_snprintf("%d", peer->listconns + peer->downloadedfiles));
}

int ftp_dumper(FTPSTATE *peer, NEWFILE *infile, int number, char *fmessage, int dotokens)
{
	char *outstr;
	OUTBUFF *outputbuffer;
	TOKENSET *ts;
	char *inp;

	if (infile != NULL)
	{
		if (dotokens)
		{
			ts = tokenset_new();
			setdumptokens(peer, ts);
		}
				
		outputbuffer = obuff_new(peer->remotefd);
		while ((inp = nfgetcs(infile, '\n')) != NULL)
		{
			if (inp[strlen(inp)-1] == '\n')
				inp[strlen(inp)-1] = 0;
			
			if (dotokens)
				inp = tokenset_apply(ts, inp);
				
			outstr = safe_snprintf("%d-%s\xD\n", number, inp);
			obuff_write(outputbuffer, outstr, strlen(outstr));
			freewrapper(inp);
			freewrapper(outstr);
		}

		nfclose(infile);
		/* Flush the output buffers */

		if (dotokens)
			tokenset_finish(ts);
		
		obuff_flush(outputbuffer);
		obuff_free(outputbuffer);
	}

	ftp_write(peer, number, fmessage);  

	return(FALSE);
}

int ftp_rnto(FTPSTATE *peer, char *filename)
{ 
	char *badfile;
	
	if (!peer->renameoldname)
		return(ftp_write(peer, 503, "Error, you must specify the source first"));
	
	dir_tovirtual(peer, peer->renameoldname);

	if ((badfile = file_rename(peer, peer->renameoldname, filename)) == NULL)
	{
		ftp_write(peer, 250, "Renaming '%s' to '%s'", peer->renameoldname,  filename);
		freewrapper(peer->renameoldname);
		peer->renameoldname = NULL;
	}
	else
		reporterror(peer, badfile, errno);

	return(FALSE);
}

int ftp_mkd(FTPSTATE *peer, char *filename)
{
	if (file_mkdir(peer, filename) == 0)
	{
		char *newfile = file_expand(peer, filename);
		dir_tovirtual(peer, newfile);
		ftp_write(peer, 257, "\x22%s\x22 was created", newfile);
		freewrapper(newfile);
	}
	else
		reporterror(peer, filename, errno); 

	return(FALSE);
}

int ftp_rmd(FTPSTATE *peer, char *filename)
{
	if (file_rmdir(peer, filename) == 0)
		ftp_write(peer, 250, "Directory '%s' removed", filename);
	else
		reporterror(peer, filename, errno); 

	return(FALSE);
}

int ftp_cwddo(FTPSTATE *peer, char *newdir, int dump)
{
	char *tmp = strdupwrapper(peer->pwd);
	
	dir_combine(peer, &tmp, newdir);
	
	if (!checkchdir(peer, tmp))
	{
		freewrapper(tmp);
		reporterror(peer, newdir, errno);
	}
	else 
	{
		NEWFILE *nfile = NULL;
		char *outstr;
		
		/* If we need to print a file, then open it */ 
		if (dump)
			nfile = nfopen(peer->cwddump);
		
		freewrapper(peer->pwd);
		peer->pwd = tmp;
		dir_tovirtual(peer, peer->pwd);
		outstr = safe_snprintf("\x22%s\x22 is the current directory.", peer->pwd);
		dir_toreal(peer, peer->pwd);
		
		ftp_dumper(peer, nfile, 257, outstr, TRUE);
		freewrapper(outstr);
	}
	
	return(FALSE);
}

int ftp_cwd(FTPSTATE *peer, char *newdir)
{
	return(ftp_cwddo(peer, newdir, TRUE));
}

int ftp_run(FTPSTATE *peer, INPUTLINE *cmd, char *token)
{
	if ((cmd->command->ftpfunc) == NULL)
		return(ftp_write(peer, 504, "%s: command not recognised.", token));
	
	if ((cmd->command->paramnum) && (cmd->parameters == NULL))
		return(ftp_write(peer, 500, "%s: parameters needed.", token));
	
	if ((cmd->command->needslogin) && (peer->loggedin != TRUE))
		return(ftp_write(peer, 530, "Please login first."));
	
	if ((!cmd->command->dataportok) && (peer->dport))
		return(ftp_write(peer, 520, "%s: command not valid during data connection.", token));
	else
		return(cmd->command->ftpfunc(peer, cmd->parameters));
}

int ftp_quit(FTPSTATE *peer, char *param)
{
	if (peer->quitdump)
	{
		NEWFILE *nfile = nfopen(peer->quitdump);
		ftp_dumper(peer, nfile, 221, "Goodbye, logging out.", TRUE);
	}
	else
		ftp_write(peer, 221, "Goodbye, logging out.");
	log_addentry(MYLOG_LOGIN, peer->remoteip, peer->username, "Logging out out (by QUIT cmd)"); 
	return(TRUE);
}

int ftp_user(FTPSTATE *peer, char *param)
{
	if (peer->jailenabled)
		return(ftp_write(peer, 530, "You cannot log on as another user."));
	if (peer->loginsleft == 0)
		return(ftp_write(peer, 530, "You have run out of login credits."));
	peer->loggedin = 0;
	freewrapper(peer->username);
	peer->username = strdupwrapper(param);
	pnums_setusername(peer->threadnum, "<unknown>");
	return(ftp_write(peer, 331, "Welcome '%s', enter password to login.", peer->username));
}

int ftp_pass(FTPSTATE *peer, char *param)
{
	int result = FALSE;
	int loginok;
	int toomany = FALSE;
	char *errmsg;
	
	if (peer->loggedin)
		return(ftp_write(peer, 530, "Already logged in!"));

	if (strcmp(peer->username, "<unknown>") == 0)
		return(ftp_write(peer, 503, "Supply a username first."));
	
	/* check password against username */
	loginok = (param != NULL);
	
	if (loginok)
	{
		errmsg = setuseropts(peer, param);
		loginok = (errmsg == NULL);
	}
	
	if (loginok)
	{
		toomany = (pnums_countuser(peer->username) >= peer->maxusers);  
		if (toomany)
			loginok = FALSE;
	}
		
	if (loginok)
	{
		char *outstr;
		NEWFILE *nfile = nfopen(peer->logindump);
		pnums_setusername(peer->threadnum, peer->username);
		
		log_addentry(MYLOG_LOGIN, peer->remoteip, peer->username, "Login successful."); 
		outstr = safe_snprintf("User '%s' login successful.", peer->username);
		peer->pwd = strdupwrapper(peer->basedir);
		dir_combine(peer, &(peer->pwd), peer->homedir);
		
		result = ftp_dumper(peer, nfile, 230, outstr, TRUE);
		freewrapper(outstr);
	}
	else
	{
		peer->loggedin = FALSE;
		log_addentry(MYLOG_LOGIN, peer->remoteip, peer->username, "Login failed");
		peer->timeout = config->timeout;
		if (toomany)
		{
			NEWFILE *nfile = nfopen(peer->busydump);
			ftp_dumper(peer, nfile, 421, "Too many users. Cannot login.", TRUE);
		}
		else
		{
			usleep(config->authwait);
			result = ftp_write(peer, 530, "Login for user '%s' failed (%s).", peer->username, errmsg);
			peer->loginsleft--;
			freewrapper(errmsg);
		}
		freewrapper(peer->username);
		peer->username = strdupwrapper("<unknown>");
		pnums_setusername(peer->threadnum, peer->username);
	}
	return(3);      /* make sure timeout gets updated */
}

int ftp_rest(FTPSTATE *peer, char *param)
{
	int newpos, res;
	res = sscanf(param, "%d", &newpos);
	if ((res != 1) && (newpos > 0))
		return(ftp_write(peer, 501, "Resume value is not valid."));
	else
	{
		peer->restartpos = newpos;
		return(ftp_write(peer, 350, "Starting at position %d.", peer->restartpos));
	}
}

int ftp_syst(FTPSTATE *peer, char *param)
{
	return(ftp_write(peer, 215, "UNIX Type: L8"));
}
	
int ftp_port(FTPSTATE *peer, char *param)
{
	if (readipstr(param, &(peer->remoteport)))
	{
		if (peer->passiveport)
		{
			select_delfd(peer->sel, peer->passiveport);
			peer->passiveport = 0;
		}
		if (peer->remoteport < 1024)
		{
			peer->remoteport = 0;
			return(ftp_write(peer, 500, "Cannot connect to ports below 1024."));
		}
		return(ftp_write(peer, 200, "PORT command successful."));
	}
	else
		return(ftp_write(peer, 501, "Invalid parameters for PORT command."));
}
	
int ftp_pwd(FTPSTATE *peer, char *param)
{
	return(ftp_cwddo(peer, ".", FALSE));
}

int ftp_type(FTPSTATE *peer, char *param)
{
	if ((param[0] & (255-32)) == 'A')
	{
		peer->binary = FALSE;
		return(ftp_write(peer, 200, "TYPE is now ASCII."));
	}
	else if ((param[0] & (255-32)) == 'I')
	{
		peer->binary = TRUE;
		return(ftp_write(peer, 200, "TYPE is now BINARY."));
	}
	else
		return(ftp_write(peer, 504, "Unimplemented TYPE."));
}

int ftp_abor(FTPSTATE *peer, char *param)
{
	if (peer->dport)
		abortdatasocket(peer);
	return(ftp_write(peer, 226, "ABOR Successful."));
}

int ftp_pasv(FTPSTATE *peer, char *param)
{
	unsigned int a1, a2, a3, a4, a5, a6;
	if (peer->passiveport != 0)
	{
		select_delfd(peer->sel, peer->passiveport);
	}
	peer->remoteport = 0;
	peer->passiveport = listenparrelelport(peer->remotefd, &(peer->remoteport), &a1, 5);
	select_addfd(peer->sel, peer->passiveport);
	a2 = ((a1 >> 8) & 255);
	a3 = ((a1 >> 16) & 255);
	a4 = ((a1 >> 24) & 255);
	a1 = a1 & 255;
	a5 = (peer->remoteport >> 8);
	a6 = (peer->remoteport & 255);
	return(ftp_write(peer, 227, "Passive mode on (%d,%d,%d,%d,%d,%d)", a4, a3, a2, a1, a5, a6));
}

int ftp_list(FTPSTATE *peer, char *param)
{
	char *p = param;
	int done = FALSE;
	int parm = 0;
	while (!done)
	{
		if (p == NULL)
			done = TRUE;
		else if ((p[0] != '-') && (p[0] != ' ') && (p[0] != 0))
			done = TRUE;
		else
		{
			parm = parm | ftplist_parseflags(p + 1);
			p = strchr(p, ' ');
			if (p != NULL)
				p++;
		}
	}

	return(ftp_lister(peer, p, FALSE, parm));
}

int ftp_nlst(FTPSTATE *peer, char *param)
{
	char *p = param;
	int done = FALSE;
	int parm = 0;
	while (!done)
	{
		if (p == NULL)
			done = TRUE;
		else if ((p[0] != '-') && (p[0] != ' ') && (p[0] != 0))
			done = TRUE;
		else
		{
			parm = parm | ftplist_parseflags(p + 1);
			p = strchr(p, ' ');
			if (p != NULL)
				p++;
		}
	}

	return(ftp_lister(peer, p, TRUE, parm));
}

int ftp_cdup(FTPSTATE *peer, char *param)
{
	return(ftp_cwd(peer, ".."));
}

int ftp_rnfr(FTPSTATE *peer, char *param)
{
	if (peer->renameoldname)
		freewrapper(peer->renameoldname);
	peer->renameoldname = file_expand(peer, param);
	return(ftp_write(peer, 350, "Source ready, waiting for destination."));
}

int ftp_noop(FTPSTATE *peer, char *param)
{
	return(ftp_write(peer, 200, "No operation was successful."));
}

int ftp_rein(FTPSTATE *peer, char *param)
{
	if (peer->jailenabled)
		return(ftp_write(peer, 530, "You cannot log on as another user."));
	peer->loggedin = FALSE;
	pnums_setusername(peer->threadnum, "<unknown>");
	abortdatasocket(peer);
	return(ftp_write(peer, 220, "Reinitialize successful, enter username."));
} 

int ftp_none(FTPSTATE *peer, char *param)
{
	return(FALSE);
}

int ftp_help(FTPSTATE *peer, char *param)
{
	return(ftp_dohelp(peer, mainftpcmd, param));
}

int ftp_dohelp(FTPSTATE *peer, FTPCMD *ftpcmds, char *command)
{
	OUTBUFF *outb;
	int result = FALSE;
	int count = 0;
	char outstr[BUFFSMALL]; /* Ok because the help never overflows it! */
	
	if (command != NULL)
	{
		while((strcasecmp(ftpcmds[count].command, command) != 0) &&
		     (ftpcmds[count].ftpfunc != NULL))
			count++;
		
		if (ftpcmds[count].ftpfunc == NULL)
			ftp_write(peer, 502, "Command '%s' not known.", command);
		else
			ftp_write(peer, 214, "%-8s : %s", ftpcmds[count].command, ftpcmds[count].helpdesc);
		return(FALSE);
	}
	
	outb = obuff_new(peer->remotefd);
	result = (-1 == obuff_writes(outb, "214-Recognized commands:"));
	while (ftpcmds[count].ftpfunc != NULL)
	{
		if (((count) % 7) == 0)
			sprintf(outstr, "\xD\n    %-9s", ftpcmds[count].command);
		else
			sprintf(outstr, "%-9s", ftpcmds[count].command);

		result = (-1 == obuff_writes(outb, outstr));
		count++;
	}
	
	result = (-1 == obuff_writes(outb, "\xD\n"));
	result = (-1 == obuff_flush(outb));
	
	if (!result)
		result = ftp_write(peer, 214, "End of help."); 

	obuff_free(outb);
	return(result);
}

int ftp_allo(FTPSTATE *peer, char *param)
{
	return(ftp_write(peer, 202, "Space allocation unnessecary."));
}

int ftp_acct(FTPSTATE *peer, char *param)
{
	return(ftp_write(peer, 200, "Account unnessecary and ignored."));
}
