/* Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "ftpd.h"

#define 	LISTALL		1
#define 	LISTDIR		2

int ftplist_parseflags(char *params)
{
	int count, output = 0;
	
	for (count = 0; count < strlen(params); count++)
	{
		if (params[count] == 'a')
			output = output | LISTALL;
		if (params[count] == 'd')
			output = output | LISTDIR;
		if (params[count] == ' ')
			return(output);
	}
	
	return(output);
}

char *listmakeline(char *fullfile, char *shortname, STRCACHE *uidcache, STRCACHE *gidcache)
{
	char *output;
	struct stat fileinfo;
	int statresult;
	char *datastr;
	
	/* make the base string */
	output = safe_snprintf("----------   0                            0 Jan  1 00:00 %s", shortname);
	statresult = lstat(fullfile, &fileinfo);
	
	if (statresult == 0)
	{
		char numstr[20];
		
		/* insert the permissions/ filetype */
		convertperms(output, fileinfo.st_mode);
		
		/* Insert links */
		sprintf(output + 11, "%3d", MINIMUM(fileinfo.st_nlink, 999));
		output[14] = 32;

		/* Insert links */
		sprintf(output + 33, "%10d", (unsigned int)fileinfo.st_size);
		output[43] = 32;

		/* Insert date */
		datastr = ctime(&(fileinfo.st_mtime));
		memcpy(output + 44, datastr + 4, 12);
	
		/* Do symbolic links */
		if (*output == 'l') 
		{
			char linkname[BUFFERSIZE];
			char *newout;
			linkname[readlink(fullfile, linkname, BUFFERSIZE - 1)] = 0;
			newout = safe_snprintf("%s -> %s", output, linkname);
			freewrapper(output);
			output = newout;
		}
	
		/* do owner */
		if ((datastr = strcache_check(uidcache, fileinfo.st_uid)) == NULL)
		{
			struct passwd *user;
			
			user = getpwuid(fileinfo.st_uid);
			if (user == NULL)
			{
				sprintf(numstr, "%d", fileinfo.st_uid);
				datastr = numstr;
			}
			else
				datastr = user->pw_name;
		
			strcache_add(uidcache, fileinfo.st_uid, datastr);	
		}
		memcpy(output + 15, datastr, MINIMUM(strlen(datastr), 8)); 					
			
		/* do group */		
		if ((datastr = strcache_check(gidcache, fileinfo.st_gid)) == NULL)
		{	
			struct group *grp;
			
			grp = getgrgid(fileinfo.st_gid);
			if (grp == NULL)
			{
				sprintf(numstr, "%d", fileinfo.st_gid);
				datastr = numstr;
			}
			else
				datastr = grp->gr_name;
			
			strcache_add(gidcache, fileinfo.st_gid, datastr);
		}
		memcpy(output + 24, datastr, MINIMUM(strlen(datastr), 8));
	}
	return(output);
}

char *listfiles(FTPSTATE *peer, char *pattern, int nlist, int *usedstring, int params)
{
	STRCACHE *uidcache;
	STRCACHE *gidcache;
	glob_t dirdata;
	char *output;
	int outlen, count, usedstr, result;
	*usedstring = 0;
	
	result = file_glob(peer, pattern, &(dirdata), params & LISTALL);
	
	if (result != 0)
	{
		if (errno != EPERM) globfree(&dirdata);
		return(strdupwrapper(""));
	}
	
	uidcache = strcache_new();
	gidcache = strcache_new();
	
	/* Initialize output string */
	output = mallocwrapper(sizeof(char) * BUFFERSIZE);
	outlen = BUFFERSIZE;
	usedstr = 0;
	
	for (count = 0; count < dirdata.gl_pathc; count++)
	{
		char *shortname;
		char *filestr;
		int filestrlen;
		
		shortname = (strrchr(dirdata.gl_pathv[count], '/') + 1);

		if (nlist)
			filestr = shortname;
		else
			filestr = listmakeline(dirdata.gl_pathv[count], shortname,
						uidcache, gidcache);
		
		filestrlen = strlen(filestr) + 3;
		while (usedstr + filestrlen > outlen)
		{
			outlen *= 2;
			if (outlen > MAXLISTSIZE)
			{
				/* list too large */
				if (!nlist)
					freewrapper(output);
				freewrapper(filestr);
				output = strdupwrapper("list overflow\r\n");
				usedstr = strlen(output);
				goto endoflist;
			}	
			reallocwrapper(outlen, (void **)&output);
		}
		
		/* put it at the end of the buffer */
		memcpy(&(output[usedstr]), filestr, filestrlen - 3);
		usedstr += (filestrlen - 1);
		output[usedstr - 1] = '\n';
		output[usedstr - 2] = '\r';
			
		if (!nlist)
			freewrapper(filestr);
	}

endoflist:

	output[usedstr] = 0;
	globfree(&dirdata);
	strcache_free(uidcache);
	strcache_free(gidcache);
	*usedstring = usedstr;
	return(output);
}				

int listdatasock_write(SELECTER *sel, void *peerv)
{
	FTPSTATE *peer = (FTPSTATE *)peerv;
	DATAPORT *dport = peer->dport;
	int size;

	size = write(dport->remotefd, &(dport->buffer[dport->pos]), 
		MINIMUM(dport->tsize - dport->pos, BUFFERSIZE));
	
	if (size > 0)
	{
		peer->listdownloadedbytes += size;
		dport->pos += size;
	}
	
	if ((size <= 0) || (dport->die) || (dport->pos >= dport->tsize))
	{
		socket_flush_wait(dport->remotefd, peer->timeout);
		close(dport->remotefd);
		
		if (dport->pos >= dport->tsize)
			ftp_write(peer, 226, "Transfer done. %d bytes transfered.", dport->tsize);
		else
			ftp_write(peer, 426, "Transfer aborted.");
		closedatasocket(peer);
		return(2);
	}
	
	return(FALSE);
}

char *listmakepattern(FTPSTATE *peer, char *parm, int flags)
{
	char *dir;
	char *parm2;
 	 	
	if (parm == NULL)
		parm2 = strdupwrapper("*");
	else
		parm2 = strdupwrapper(parm);

	dir = file_expand(peer, parm2);
	freewrapper(parm2);

	/* This will check for directory names and changes them to the files
	   in the directory instead unless user specifiess -d */
	if (checkdir(dir) && !(flags & LISTDIR))
	{	
		int a = strlen(dir);
		reallocwrapper(a + 6, (void *)&dir);
		dir[a] = '/'; dir[a+1] = '*'; dir[a+2] = 0;
	}
	
	return(dir);
}

int ftp_lister(FTPSTATE *peer, char *parm, int nlist, int params)
{
	char *dir;
	char *buffer;
 	int  size;
 	 	
 	dir = listmakepattern(peer, parm, params);
 	
	/* create the listing */
	buffer = listfiles(peer, dir, nlist, &size, params); 
	peer->listconns++;
	
	if (startdatasocket(peer, -1, NULL) == -1)
	{
		freewrapper(buffer);
		reporterror(peer, "file list", errno);
	}
	else
	{
		peer->dport->firstop = listdatasock_write;
		peer->dport->tsize = size;
		
		/* replace the standard buffer with ours */
		freewrapper(peer->dport->buffer);
		peer->dport->buffer = buffer;
		peer->dport->filter = NULL;
	}
	
	freewrapper(dir);
	return(FALSE);
}
