/* Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "ftpd.h"
#include "ftpcmd.h"

extern FTPCMD siteftpcmd[];

int ftp_site(FTPSTATE *peer, char *cmd)
{
	int result = FALSE;
	INPUTLINE acmd;
	
	/* if we were invoked without parameters */
	if (cmd == NULL)
		return(ftp_write(peer, 500, "SITE requires parameters. Try SITE HELP."));
	
	/* separate the site command from the site command parameters */
	cmd_split(peer, &acmd, cmd, siteftpcmd, FALSE);
	
	result = ftp_run(peer, &acmd, "SITE");
	
	if (acmd.parameters != NULL)
		freewrapper(acmd.parameters);

	return(result);
}

int ftpsite_dump(FTPSTATE *peer, char *cmd)
{
	NEWFILE *nfile = file_nfopen(peer, cmd);
	if (nfile != NULL) /* Log the transfer */
	{
		char *lfile = file_expand(peer, cmd);
		char *outstr = safe_snprintf("dump %s", lfile);
		log_addentry(MYLOG_FTRANS, peer->remoteip, peer->username, outstr);
		freewrapper(lfile);
		freewrapper(outstr);
	}
	return(ftp_dumper(peer, nfile, 200, "SITE DUMP done!", FALSE));
}

int ftpsite_umask(FTPSTATE *peer, char *cmd)
{
	int result = FALSE;
	int newumask = 0;
	
	if (peer->chmodable && cmd)
	{
		int res = sscanf(cmd, "%o", &newumask);

		newumask &= 0777;
		if (res == 0)
			result = ftp_write(peer, 501, "SITE UMASK requires octal umask to be supplied.");
		else
		{
			result = ftp_write(peer, 200, "New UMASK is %03o (old was %03o)", newumask, peer->umask);
			peer->umask = newumask;
		}
	}
	else
	{
		result = ftp_write(peer, 200, "Current UMASK is %03o", peer->umask);
	}
	return(result);
}
	
int ftpsite_help(FTPSTATE *peer, char *cmd)
{
	return(ftp_dohelp(peer, siteftpcmd, cmd));
}

int ftpsite_idle(FTPSTATE *peer, char *cmd)
{
	int newtimeout;
	
	if (!cmd)
		return(ftp_write(peer, 200, "Timeout is set to %d seconds.", peer->timeout));

	if (peer->maxtimeout < 30)
		return(ftp_write(peer, 500, "Timeout too small to be adjusted. (%d sec)", peer->maxtimeout));

	if (sscanf(cmd, "%d", &newtimeout) < 1)
		return(ftp_write(peer, 500, "Timeout must be an integer value."));
	
	if ((newtimeout >= 30) && (newtimeout <= peer->maxtimeout))
	{
		peer->timeout = newtimeout;
		ftp_write(peer, 200, "Timeout set to %d seconds.", peer->timeout);
	}
	else
		return(ftp_write(peer, 500, "Timeout must be between %d and %d seconds.", 30, peer->maxtimeout));

	return(3); /* make sure timeout gets updated! */
}

int ftpsite_chmod(FTPSTATE *peer, char *cmd)
{
	int result = FALSE;
	
	if (peer->chmodable)
	{
		int newmode;
		char *mode = NULL;
				
		if (cmd != NULL)
			mode = getkeyword(&cmd);

		if (cmd == NULL)
			result = ftp_write(peer, 501, "SITE CHMOD expects mode and filename parameters.");
		else if (sscanf(mode, "%o", &newmode) == 0)
			result = ftp_write(peer, 501, "SITE CHMOD requires octal mode number");
		else
		{
			newmode &= 0777;
			result = ftp_chmod(peer, cmd, newmode);
		}
				
		if (mode)
			freewrapper(mode);
	}
	else
		result = ftp_write(peer, 500, "SITE CHMOD access denied.");
	
	return(result);
}

int ftpsite_access(FTPSTATE *peer, char *cmd)
{
	int count = 0;
	int result;
	OUTBUFF *outb = obuff_new(peer->remotefd);
	char *outstr = mallocwrapper(BUFFERSIZE);
	char *outstr2;
	
	sprintf(outstr, "211-Access lists for user %s:\xD\n", peer->username);
	result = (-1 == obuff_writes(outb, outstr)); 
	sprintf(outstr, "    ***Path/Filename***                                          ***Access***\xD\n\xD\n");
	result = (-1 == obuff_writes(outb, outstr)); 
	
	while((peer->acldata[count].pathlen) > 0)
	{
		outstr2 = safe_snprintf("    %-60s %s\xD\n", peer->acldata[count].path, peer->acldata[count].perms);
		result = (-1 == obuff_writes(outb, outstr2)); 
		count++;
		freewrapper(outstr2);
	}

	result = (-1 == obuff_flush(outb));
	obuff_free(outb);
	ftp_write(peer, 211, "End of access list.");
	freewrapper(outstr);
	return(result);
}
