/* Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "ftpd.h"

int ftp_stat(FTPSTATE *peer, char *param)
{
	int result = FALSE;
	char *outstr = mallocwrapper(BUFFERSIZE);
	OUTBUFF *outb = obuff_new(peer->remotefd);
	char *outstr2;
	
	if (param == NULL) 
	{
		char m;
		unsigned int ip = peer->remoteip;
		
		if(peer->binary)
			m = 'I';
		else
			m = 'A';
		dir_tovirtual(peer, peer->pwd);
		
		sprintf(outstr, "211-Server is running "PROGNAME" ("VERSTR")\xD\n");
		result = (-1 == obuff_writes(outb, outstr)); 
		outstr2 = safe_snprintf("    Username: %s\xD\n    Current Directory: %s\xD\n", peer->username, peer->pwd);
		result = (-1 == obuff_writes(outb, outstr2)); 
		freewrapper(outstr2);
		sprintf(outstr, "    File transfer mode: %c\xD\n    Restart position: %d\xD\n", m, peer->restartpos);
		result = (-1 == obuff_writes(outb, outstr)); 
		sprintf(outstr, "    Umask of new files and dirs: %04o\xD\n", peer->umask);
		result = (-1 == obuff_writes(outb, outstr)); 
		sprintf(outstr, "    Timeout: %d seconds\xD\n", peer->timeout);
		result = (-1 == obuff_writes(outb, outstr)); 
		sprintf(outstr, "    Your IP: %d.%d.%d.%d\xD\n", ip >> 24, (ip >> 16) & 255, (ip >> 8) & 255, ip & 255);
		result = (-1 == obuff_writes(outb, outstr)); 
		if (peer->dport)
		{
			sprintf(outstr, "    Data Connection in progress: ");
			result = (-1 == obuff_writes(outb, outstr));
			if (!peer->dport->graceful)
				sprintf(outstr, "Connection is not yet completed.\xD\n");
			else if (peer->dport->tsize == -1)
				sprintf(outstr, "Data Transfered: %d\xD\n", MAXIMUM(0, peer->dport->pos - peer->dport->startpos));
			else
				sprintf(outstr, "Data Transfered: %d of %d\xD\n", MAXIMUM(0, peer->dport->pos - peer->dport->startpos), peer->dport->tsize);
			result = (-1 == obuff_writes(outb, outstr));
		}
		dir_toreal(peer, peer->pwd);
	}
	else 
	{
		/* do a listing */
		char *cmd = getkeyword(&param);
		int buflen, ressize, bufpos = 0;
		char *pattern = (char *)listmakepattern(peer, cmd, 0);
		char *buffer = (char *)listfiles(peer, pattern, FALSE, &buflen, 0);
		char *out2 = safe_snprintf("211-Status of %s:\r\n", cmd);
			 
		ressize = write(peer->remotefd, out2, strlen(out2));
		while ((bufpos < buflen) && (ressize > 0))
		{
			ressize = write(peer->remotefd, buffer + bufpos, MINIMUM(buflen - bufpos, BUFFERSIZE));
			bufpos+= ressize;
		}
		freewrapper(out2);
		freewrapper(pattern);
		freewrapper(buffer);
		freewrapper(cmd);		
	}
	result = (-1 == obuff_flush(outb));
	obuff_free(outb);
	ftp_write(peer, 211, "End of status.");
	freewrapper(outstr);
	return(result);
}
