/* Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "ftpd.h"
#include "ftpcmd.h"

extern FTPCMD ftpcommandtable[];

int filter_toascii(char *data, int *len)
{
	int count, len2 = *len;
	char buffer2[BUFFERSIZE * 2];
	int last = 0;
	char *b2ptr = buffer2;
	
	for (count = 0; count < len2; count++)
	{
		if ((data[count] == 10) && (last != 13))
		{
			*b2ptr = 13;
			b2ptr++;
			(*len)++;
		}
		*b2ptr = data[count];
		last = *b2ptr;
		b2ptr++;
	}
	memcpy(data, buffer2, *len);
	return(*len);
}

int filter_fromascii(char *data, int *len)
{
	char *dptr = data;
	int count, len2 = *len;
	
	for (count = 0; count < len2; count++)
	{
		if (data[count] != 13)
		{
			*dptr = data[count];
			dptr++;
		}
		else
			(*len)--;
	}
	return(len2);
}
 
int datasock_read(SELECTER *sel, void *peerv);

int datasock_write(SELECTER *sel, void *peerv)
{
	FTPSTATE *peer = (FTPSTATE *)peerv;
	DATAPORT *dport = peer->dport;
	int size;

	size = dport->pos - dport->startpos;
	
	if ((size >= 0) && (size < dport->buffersize))
	{
		size = dport->buffersize -  size;

		dport->bufferpos = dport->buffer + size;
		dport->buffersize -= size;
	} 
	else if (size < 0)
	{
		/* throw it all away */
		dport->buffersize = 0;
	}
	
	size = write(*(dport->fdout), dport->bufferpos, dport->buffersize);
	
	if (size < 0)
	{
		dport->die = TRUE;
		dport->buffersize = 0;
	}
	else
	{
		dport->buffersize -= size;
		dport->bufferpos += size;
		if (dport->remotefd == *(dport->fdout))
			peer->downloadedfilebytes += size;
	}
	
	if (dport->buffersize == 0)
	{
		select_takewrite(sel, *(peer->dport->fdout));
		select_addread(sel, *(peer->dport->fdin), datasock_read, peer);
	}
	return(FALSE);
}
			
int datasock_read(SELECTER *sel, void *peerv)
{
	FTPSTATE *peer = (FTPSTATE *)peerv;
	DATAPORT *dport = peer->dport;
	int size;
	
	size = dport->buffersize = read(*(dport->fdin), dport->buffer, BUFFERSIZE);

	select_takeread(sel, *(dport->fdin));
	
	if ((size <= 0) || (dport->die))
	{
		select_delfd(sel, *(dport->fdout));
		if ((size == 0) && (!dport->die))
			ftp_write(peer, 226, "Transfer done. %d bytes transfered", MAXIMUM(0, peer->dport->pos - peer->dport->startpos));
		else
			ftp_write(peer, 426, "Transfer aborted. %d bytes transfered", MAXIMUM(0, peer->dport->pos - peer->dport->startpos));
		closedatasocket(peer);
		return(2);
	}
	else
	{
		select_addwrite(sel, *(dport->fdout), datasock_write, peer);
		if (dport->remotefd == *(dport->fdin))
			peer->uploadedfilebytes += size;
	}
	
	dport->bufferpos = dport->buffer;
	dport->pos += dport->filter(dport->buffer, &(dport->buffersize));
	
	return(FALSE);
}
	
int ftp_retr(FTPSTATE *peer, char *filename)
{
	int filefd;
	
	if ((filefd = file_open(peer, filename, O_RDONLY)) < 0)
		reporterror(peer, filename, errno);
	else
	{
		peer->downloadedfiles++;
		if (startdatasocket(peer, filefd, filter_toascii) == -1)
		{
			close(filefd);
			reporterror(peer, "data socket", errno);
		}
		else
		{
			char *logdat, *fullname = file_expand(peer, filename);
			logdat = safe_snprintf("retrieve %s", fullname);
			log_addentry(MYLOG_FTRANS, peer->remoteip, peer->username, logdat);
			freewrapper(logdat);
			freewrapper(fullname);
		
			peer->dport->fdout = &(peer->dport->remotefd);
			peer->dport->fdin = &(peer->dport->filefd);
		
			if (peer->binary)
			{
				int size = lseek(filefd, 0, SEEK_END);
				lseek(filefd, MINIMUM(size, peer->dport->startpos), SEEK_SET);
				peer->dport->pos = MINIMUM(size, peer->dport->startpos);
				peer->dport->tsize = size - peer->dport->startpos;
			}
			peer->dport->firstop = datasock_read;
		}
	}
	return(FALSE);
}

int ftp_stor(FTPSTATE *peer, char *filename)
{
	int filefd;
	char *fullname;

	/* There is 2 good reasons thus is here:
		1) My code won't work for resume ASCII upload.
		2) It is impossible to do. Telnet ascii to unix ascii is
		   irreversible. Therefor you don't know where the user
		   wants to start saving.
	*/
	if ((peer->restartpos != 0) && (!peer->binary))
		return(ftp_write(peer, 500, "Cannot resume ASCII upload. Use APPE!."));
	
	fullname = file_expand(peer, filename);

	if (checkfile(fullname))
	{
		if (peer->restartpos == 0)
			filefd = file_creat(peer, filename);
		else
			filefd = file_open(peer, filename, O_WRONLY);
	}
	else 
		filefd = file_creat(peer, filename);
	
	if (filefd < 0)
		reporterror(peer, filename, errno);
	else
	{
		peer->uploadedfiles++;
		if (startdatasocket(peer, filefd, filter_toascii) == -1)
		{
			close(filefd);
			reporterror(peer, "data socket", errno);
		}
		else
		{
			char *logdat;
			logdat = safe_snprintf("store %s", fullname);
			log_addentry(MYLOG_FTRANS, peer->remoteip, peer->username, logdat);
			freewrapper(logdat);
			
			peer->dport->fdout = &(peer->dport->filefd);
			peer->dport->fdin = &(peer->dport->remotefd);
		
			if (peer->binary)
			{
				lseek(filefd, peer->dport->startpos, SEEK_SET);
				peer->dport->pos = peer->dport->startpos;
			}
				
			peer->dport->tsize = -1;
			peer->dport->firstop = datasock_read;
			
		}
	}
	
	freewrapper(fullname);
	return(FALSE);
}

int ftp_stou(FTPSTATE *peer, char *filename)
{
	char *fullname = file_expand(peer, filename);
	int epos = strlen(fullname);
	int pos = 0;
	
	reallocwrapper(epos + 5, (void *)&fullname);
	
	while(checkfile(fullname) && (pos <= 1000))
	{
		fullname[epos] = '.';
		fullname[epos+1] = (pos / 100) + '0';
		fullname[epos+2] = (pos % 100) / 10 + '0';
		fullname[epos+3] = (pos % 10) + '0';
		fullname[epos+4] = 0;
		pos++;
	}
	
	if (pos > 1000)
		pos = ftp_write(peer, 500, "Could not determine a unique filename!");
	else
	{
		dir_tovirtual(peer, fullname);
		pos = ftp_stor(peer, fullname);
	}
	
	freewrapper(fullname);
	return(pos);
}

int ftp_appe(FTPSTATE *peer, char *filename)
{
	int filefd;
	char *fullname = file_expand(peer, filename);

	peer->restartpos = 0;
	
	if (checkfile(fullname))
		filefd = file_open(peer, filename, O_WRONLY);
	else 
		filefd = file_creat(peer, filename);
	

	if (filefd < 0)
		reporterror(peer, filename, errno);
	else
	{
		peer->uploadedfiles++;
		if (startdatasocket(peer, filefd, filter_toascii) == -1)
		{
			close(filefd);
			reporterror(peer, "data socket", errno);
		}
		else
		{
			char *logdat;
			logdat = safe_snprintf("append %s", fullname);
			log_addentry(MYLOG_FTRANS, peer->remoteip, peer->username, logdat);
			freewrapper(logdat);
			
			peer->dport->fdout = &(peer->dport->filefd);
			peer->dport->fdin = &(peer->dport->remotefd);
		
			lseek(filefd, 0, SEEK_END); /* set to append */
			peer->dport->pos = 0;

			peer->dport->tsize = -1;
			peer->dport->firstop = datasock_read;
			
		}
	}
	freewrapper(fullname);
	return(FALSE);
}
