/* Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "ftpd.h"

/* Initalializes the program, and reads configuration */

void checkoldinstance(void)
{
	FILE *pidfile = fopen(config->pidfile, "r");
	
	if (pidfile == NULL)
	{
		if (errno == ENOENT)
			return;	
		else
			ERRORMSGFATAL("Cannot access pid file, cannot start "PROGNAME"!");
	}
	else
	{
		int pid;
		int sres = fscanf(pidfile, "%d", &pid);
		fclose(pidfile);
		
		if(sres != 1)
			return;
		else
		{
			int result = kill((pid_t)pid, 0);
			if (result == -1)
			{
				if (errno == ESRCH)
					return;
				else
					ERRORMSGFATAL(PROGNAME" already running!");
			}
			else
				ERRORMSGFATAL(PROGNAME" already running!");
		}
	}
}

int ftpd_doconfig(char *cmd, char *data, int line, void *d)
{
	if (strcasecmp(cmd, "ftpport") == 0)
	{
		char *bindip = strchr(data, '/');
		unsigned int bip;
		
		if (bindip == NULL)
			getnetworkint("0.0.0.0", &bip);
		else
		{
			*bindip = 0;
			getnetworkint(++bindip, &bip);
		}
		
		config->listenportlen++;
		
		reallocwrapper(sizeof(int) * config->listenportlen,
				(void *)&(config->listenport));
 		reallocwrapper(sizeof(int) * config->listenportlen,
 				(void *)&(config->listenips));
 		sscanf(data, "%d", &(config->listenport[config->listenportlen - 1]));
 		config->listenips[config->listenportlen - 1] = bip;
 	}
 	else if (strcasecmp(cmd, "maxusers") == 0)
 		sscanf(data, "%d", &(config->maxthreads));
 	else if (strcasecmp(cmd, "logstrength") == 0)
 		sscanf(data, "%d", &(config->loglevel));
	else if (strcasecmp(cmd, "runasuser") == 0)
	{
		struct passwd *userinfo;
		userinfo = getpwnam(data);
		if (userinfo == NULL)
		{
			char *errmsg = safe_snprintf("runasuser: username '%s' doesn't exist!", data);
			ERRORMSGFATAL(errmsg);	
		}
		config->asgid = userinfo->pw_gid;
		config->asuid = userinfo->pw_uid;
		freewrapper(config->username);
		config->username = strdupwrapper(data);
	}
	else if (strcasecmp(cmd, "logfile") == 0)
	{
		freewrapper(config->logfile);
		config->logfile = strdupwrapper(data);
 	}
	else if (strcasecmp(cmd, "accessdir") == 0)
	{
		freewrapper(config->accessdir);
		config->accessdir = strdupwrapper(data);
 	}
 	else if (strcasecmp(cmd, "timeout") == 0)
 		sscanf(data, "%d", &(config->timeout));
	else if (strcasecmp(cmd, "addhost") == 0)
	{
		char *netmask = strchr(data, '/');
		
		if (netmask)
		{
			*netmask = 0;
			netmask++;
		}
		reallocwrapper(sizeof(long int) * (config->iplistlen + 1), (void *)&(config->iplist));
		reallocwrapper(sizeof(long int) * (config->iplistlen + 1), (void *)&(config->netmasklist));
		getnetworkint(data, &config->iplist[config->iplistlen]);
		if (netmask)
			getnetworkint(netmask, &config->netmasklist[config->iplistlen]);
		else
			getnetworkint("255.255.255.255", &config->netmasklist[config->iplistlen]);	
		config->iplistlen++;
	} 
	else if (strcasecmp(cmd, "logindump") == 0)
	{
		freewrapper(config->prelogondump);
		config->prelogondump = strdupwrapper(data);
	}
	else if (strcasecmp(cmd, "pidfile") == 0)
	{
		freewrapper(config->pidfile);
		config->pidfile = strdupwrapper(data);
 	}
	else if (strcasecmp(cmd, "email") == 0)
	{
		freewrapper(config->email);
		config->email = strdupwrapper(data);
 	}
 	else if (strcasecmp(cmd, "badauthwait") == 0)
 	{
 		sscanf(data, "%d", &(config->authwait));
 		config->authwait *= 1000;
 	}
 	else if (strcasecmp(cmd, "logintrys") == 0)
	{
 		sscanf(data, "%d", &(config->logincount));
 		if (config->logincount == 0)
 			config->logincount = -1;
 	}
 	else
 	{
 		char *errmsg = safe_snprintf("Unknown option %s on line %d in config file!", cmd, line);
 		ERRORMSGFATAL(errmsg);
 	}
	return(TRUE);
}	

void ftpd_setnogroups(void)
{
	setgroups(0, NULL);
}

void ftpd_preinit(void)
{
	config = NULL;
	pnums = NULL;
}

void ftpd_loadconfig(char *inidata)
{
	int configok;
	struct passwd *userinfo;
	
	config->parentpid = getpid(); 

	userinfo = getpwnam("nobody");
	if (userinfo == NULL)
		ERRORMSGFATAL("Cannot find uid/gid for user nobody!");

	config->nobodygid = userinfo->pw_gid;
	config->nobodyuid = userinfo->pw_uid;

#ifndef QUIET
	printf("LOADING CONFIG FILE: %s\n", inidata);
#endif
	config->maxthreads = 30;
	config->listenport = NULL;
	config->listenportlen = 0;
	config->listenfds = NULL;
	
	config->loglevel = 1;		/* File Transfers are logged */

	config->iplistlen = 0;
	config->iplist = NULL;
	config->netmasklist = NULL;
	
 	config->logfile = strdupwrapper("/var/log/muddleftpd.log");
	config->accessdir = strdupwrapper("/etc/muddleftpd");
	config->pidfile = strdupwrapper("/var/run/muddleftpd.pid");
	config->email = strdupwrapper("<unknown>");
	
	config->timeout = 300; 		/*  Timeout in 5 minutes */
	
	config->parentpid = getpid(); 
	config->asgid = getgid();
	config->asuid = getuid();
	config->rootmode = (getuid() == 0);
	config->prelogondump = strdupwrapper("");
	config->username = strdupwrapper("");
	config->authwait = 1000000;
	config->logincount = 3;
	
	configok = loadconfigfile(inidata, ftpd_doconfig, NULL);
	 	
	if (!configok)
		ERRORMSGFATAL("Cannot open config file");

	if (config->asuid != 0)
		config->rootmode = FALSE;
		
#ifndef QUIET
	if (config->rootmode)
		printf("Running as root, using full capibilities.\n");
	else
		printf("Running as normal user, change uid not possible.\n");
#endif

}

void ftpd_checkconfig(void)
{
 	/* CHECK THE SETTINGS THE USER GAVE US */
 	
 	if (config->maxthreads <= 0)
		ERRORMSGFATAL("maxusers is not valid (out of range)");
	
	if (config->listenportlen == 0)
		ERRORMSGFATAL("ftpport has not been specified.");
	
	if ((config->loglevel < 0) || (config->loglevel > 63))
		ERRORMSGFATAL("logstrength is not valid (out of range)");
	
	if (config->iplistlen == 0)
		ERRORMSGFATAL("addhost directive needed, no rules defined.");
	
	if (!checkdir(config->accessdir))
		ERRORMSGFATAL("accessdir does not exist or cannot be opened.");
	if (!config->pidfile)
		ERRORMSGFATAL("pidfile is not defined. Cannot continue!.");
}

void ftpd_init(char *filename) 
{
	if (filename == NULL)
		filename = CONFIGFILE;
	
#ifdef DEBUG
	printf("[ftpd_init filename: %s]\n", filename);
#endif
	ftpd_preinit();
	
	config = mallocwrapper(sizeof(CONFIGDATA)); 
	
#ifndef QUIET
	printf("Starting "PROGNAME" server. (Version "VERSTR")\n");
	printf("Copyright (C) 1999 Beau Kuiper\n");
	printf("License under GPL. See the http://www.gnu.org for more info.\n\n");
#endif	
 
 	ftpd_loadconfig(filename);
	ftpd_checkconfig();

	checkoldinstance();
	
	log_init();	
	shmem_init(filename);
	pnums_init();
	
	ftpd_setnogroups();
	config->umask = umask(0); /* Set umask to nothing and let me handle masking :) */
}
