/* Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "ftpd.h"

void log_init(void)
{
#if DEBUG
	printf("[log_init filename %s]\n", config->logfile);	
#endif
	config->logout = open(config->logfile, O_APPEND | O_CREAT | O_WRONLY, 0600);
	if (config->logout == -1)
	{
		ERRORMSGFATAL("Cannot open log file!");
	}
} 

void log_addentry(int type, unsigned int ip, char *username, char *desc)
{
#if DEBUG
	printf("[log_addentry entrytype: %d ip: %d user : %s desc : %s]\n", type, ip, username, desc);
#endif	
	if ((config->logout != -1) && (((config->loglevel) & type) == type))
	{
		int currenttime = time(NULL);
		char *timestr = ctime((time_t *)&currenttime);
		char ipstring[50];
		char *outstring;
		int writeresult;
#if DEBUG
		printf("[log_addentry entry added]\n");		
#endif		
		timestr[strlen(timestr) - 1] = '\0';

		sprintf(ipstring, "%u.%u.%u.%u", (unsigned char)(ip >> 24),
				       (unsigned char)((ip >> 16) & 255),
				       (unsigned char)((ip >> 8) & 255),
				       (unsigned char)((ip) & 255));
		if (type == MYLOG_DACCESS) /* access denied */
			outstring = safe_snprintf("-- %s : %s : Access Denied to ftp server.\n", timestr, ipstring);
		if (type == MYLOG_FTRANS) /* transfer log */
			outstring = safe_snprintf("++ %s : %s : %s : %s\n", timestr, ipstring, 
						username, desc);
		if (type == MYLOG_COMMAND) /* command log */
			outstring = safe_snprintf("<< %s : %s : %s : %s\n", timestr, ipstring, 
						username, desc);
		if (type == MYLOG_RESPONSE) /* response log */
			outstring = safe_snprintf(">> %s : %s : %s : %s\n", timestr, ipstring, 
						username, desc);
		if (type == MYLOG_LOGIN) /* response log */
			outstring = safe_snprintf("!! %s : %s : %s : %s\n", timestr, ipstring, 
						username, desc);
		if (type == MYLOG_INFO) /* info log */  
			outstring = safe_snprintf("** %s : Info - %s\n", timestr, desc); 
		
		writeresult = write(config->logout, outstring, strlen(outstring));
		if (writeresult < strlen(outstring))
			ERRORMSGFATAL(strerror(errno));
		
		freewrapper(outstring);
	}
} 

void log_shutdown(void)
{
	if (config->logout != -1)
		close(config->logout);
}
