/* Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "ftpd.h"

extern int nummalloc;		/* number of areas malloced at once */
int signumber;

void create_pidfile(void)
{
	FILE *newfile;
	int fd;
#ifdef DEBUG	
	printf("[create_pidfile]\n");
#endif	
	assert(config->pidfile != NULL); /* we MUST make a pid file! */
	
	fd = open(config->pidfile, O_CREAT | O_TRUNC | O_WRONLY, 0644);
		
	if (fd == -1)
		ERRORMSGFATAL("Failed to create pid file!");
		
	if (!(newfile = fdopen(fd, "w")))
		ERRORMSGFATAL("Failed to create pid file!");
		
	fprintf(newfile, "%d", (int)config->parentpid);
	fclose(newfile);
}

void delete_pidfile(void)
{
#ifdef DEBUG
	printf("[delete_pidfile]\n");
#endif
	if (config->pidfile)
		unlink(config->pidfile);
}

void sighandler(int signum)
{
	signumber = signum;
#if DEBUG
	printf("[main got a signal %d]\n", signum);
#endif
	if (signum == SIGCHLD)
	{
		int errno2, result; 
		pid_t deadpid;
#if DEBUG
		printf("[main waiting for child!]\n");		 
#endif
		errno2 = errno;
		while((deadpid = waitpid(0, &result, WNOHANG)) > 0)
			pnums_free_bypid(deadpid);
		errno = errno2;
#if DEBUG
		printf("[main child %d finished!]\n", (int)deadpid);
#endif		
		signal(SIGCHLD, sighandler);
	}
	else
	if (signum == SIGUSR1)
	{
#if DEBUG
		struct mallinfo minfo = mallinfo();
		printf("PID: %5d   Allocated memory areas: %5d Total memory allocated %5d\n", getpid(), nummalloc, minfo.uordblks);
#endif		
		signal(SIGUSR1, sighandler);
	}
}

int mainprog(int argc, char **argv)
{
	int count, signum = 0;
	SELECTER *mainports = select_new();
	
	setsid();
	
	signal(SIGIO, SIG_IGN);
	signal(SIGPIPE, SIG_IGN);
	signal(SIGTERM, sighandler);
	signal(SIGHUP, sighandler);
	signal(SIGCHLD, sighandler);
	signal(SIGUSR1, sighandler);
	signal(SIGINT, sighandler);
		
	if (argc > 1)
		ftpd_init(argv[1]);
	else
		ftpd_init(NULL);
	
	/* Listen to all the input sockets, they are changed to the fd */
	
	config->listenfds = mallocwrapper(sizeof (int) * config->listenportlen);
	for (count = 0; count < config->listenportlen; count++)
	{
		config->listenfds[count] = inport_bind(config->listenport[count], config->listenips[count]);
		select_addfd(mainports, config->listenfds[count]);
		select_addread(mainports, config->listenfds[count], 
			inport_getconn, (void *)&(config->listenfds[count]));
	}
	
	if ((config->asuid != getuid()) || (config->asgid != getgid()))
	{
		if (getuid() == 0)
		{
#ifndef QUIET
			printf("Switching to user '%s' (UID: %d, GID %d)\n", 
				config->username,
				config->asuid, config->asgid);
#endif	
			/* Tell the parent the terminal can be given back! */
			/* It must be done before we change uid because we
			   lose permission to do so after the setuid! */
			kill(getppid(), SIGHUP);

			/* Changing user to the one the user wanted */
			setgid(config->asgid);
			setuid(config->asuid);
		}
	}
	else
	{
		/* Tell the parent the terminal can be given back! */
		kill(getppid(), SIGHUP);
	}
	
	log_addentry(MYLOG_INFO, 0, NULL, PROGNAME" ("VERSTR") server started. Waiting on connections.");		

	/* Ok, now everything is ready, we can create a PID file */
	
	create_pidfile();
	
	while (signum != SIGTERM)
	{	
		select_do(mainports, &signum, -1);

		if (signum == SIGHUP)
		{
		}	 
		signal(SIGIO, SIG_IGN);
		signal(SIGHUP, sighandler);
	}

	shmem_finish();
	delete_pidfile();
	
	select_shutdown(mainports);
	
	pnums_killchildren();   /* hehehe */
	
	log_addentry(MYLOG_INFO, 0, NULL, PROGNAME" ("VERSTR") server shutting down.");	

	ftpd_shutdown();
	
	exit(0);
}

void diehandler(int signum)
{
#ifdef DEBUG
	printf("[teminal holder got signal %d]\n", signum); 
#endif
	exit(0);
}

int main(int argc, char **argv)
{
	pid_t forkresult;

	signal(SIGCHLD, diehandler);
	signal(SIGHUP, diehandler); 

        forkresult = forkwrapper();
	
	if ((int)forkresult == 0)
		mainprog(argc, argv);
		
	/* wait forever for the child to die or give us a Hangup signal */
	
	while(TRUE)
		pause();
	
	return(0);
}
