/* Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "ftpd.h"

/* #define SHOWMALLOC */
 
/* Wapper for malloc that syslogs and then dies if malloc fails before
   damage is done. Also improves performace by demanding reasonable blocks
   are allocated and realloc won't have to move it or resize it. At the
   moment at least 128 bytes are allocated all the time */
   
#define ROUNDUP(x) (x)

/*#define ROUNDUP(x)	((x) & 0xFFFFFF80) + 128 */

int nummalloc = 0;

void *mallocwrapper(int size)
{
	void *outmem = (void *)malloc(ROUNDUP(size));

	nummalloc++;

#ifdef SHOWMALLOC
	printf("malloc: %d\n", nummalloc);
#endif
	
	if (outmem == NULL)
	{
		fprintf(stderr, PROGNAME": malloc error, out of memory\n");
		kill(getpid(), SIGKILL);
	}
	return(outmem);
}

/* Wapper for remalloc that syslogs and then dies if malloc fails before
   damage is done */

void reallocwrapper(int size, void **inarea)
{
	void *outmem;

	assert(size > 0);
	
	if (*inarea == NULL)
	 	outmem = mallocwrapper(size);
	else
		outmem = (void *)realloc(*inarea, ROUNDUP(size));
	
	if (outmem == NULL)
	{
		fprintf(stderr, PROGNAME": remalloc error, out of memory\n");
		kill(getpid(), SIGKILL);
	}
#if DEBUG
	if (*inarea == outmem)
	{
		outmem = mallocwrapper(size);  /* make sure memory is moved */
		memmove(outmem, *inarea, size);
		freewrapper(*inarea);     /* free the original area */
	}
#endif
	*inarea = outmem;
}
   
char *strdupwrapper(char *s)
{
	char *outstr;
	
	assert(s != NULL);
	
	outstr = malloc(ROUNDUP(strlen(s) + 2));
	
	if (outstr == NULL)
	{
		fprintf(stderr, PROGNAME": strdup error, out of memory\n");
		kill(getpid(), SIGKILL);
	}
	nummalloc++;

#ifdef SHOWMALLOC
	printf("malloc: %d\n", nummalloc);
#endif
	
	strcpy(outstr, s);
	
	return(outstr);
}

pid_t forkwrapper(void)
{
	pid_t result = fork();
	
	if ((int)result == -1)
	{
		fprintf(stderr, PROGNAME": fork error, cannot fork\n");
		kill(getpid(), SIGKILL);
	}
	return(result);
}

void freewrapper(void *tofree)
{
	assert(tofree != NULL);
	
	free(tofree);
	
#ifdef SHOWMALLOC
	printf("free: %d\n", nummalloc);
#endif
	nummalloc--;
}

#ifndef HAVE_MEMMOVE

char *memmove(void *dest, void *src, int n)
{
	void *tmparea = mallocwrapper(n);
	
	assert(dest != NULL);
	assert(src != NULL);
	assert(n >= 0);
	
	memcpy(tmparea, src, n);
	memcpy(dest, tmparea, n);
	freewrapper(tmparea);
	
	return(dest);
}

#endif
