/* Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "ftpd.h"

int readbuffer(NEWFILE *file, int maxsize)
{
	int size;
	
	reallocwrapper(file->bufferlen + maxsize, (void *)&file->buffer);
	size = read(file->fd, &(file->buffer[file->bufferlen]), maxsize);
	if (size > 0)
		file->bufferlen += size;	
	else if (size == 0)
		file->eof = TRUE;
	else if (size == -1)
	{
		errormsg(strerror(errno), __FILE__, __LINE__);
		file->eof = TRUE;
	}
	return(size);
}
	
NEWFILE *nfopen(char *filename)
{
	NEWFILE *new;
	
	/* if we get null, return null! */
	if (!filename)
		return(NULL);
		
	if (!checkfile(filename))
		return(NULL);
		
	new = mallocwrapper(sizeof(NEWFILE));
	
	new->fd = open(filename, O_RDONLY);

	if (new->fd == -1)
	{
		freewrapper(new);
		return(NULL);
	}
			
	new->buffer = mallocwrapper(1024);
	new->bufferlen = 0;	
	new->eof = FALSE;
	return(new);	
}

NEWFILE *nfdopen(int fd)
{
	NEWFILE *new = mallocwrapper(sizeof(NEWFILE));
	
	new->fd = fd;
	
	if (new->fd == -1)
	{
		freewrapper(new);
		return(NULL);
	}
			
	new->buffer = mallocwrapper(1024);
	new->bufferlen = 0;	
	new->eof = FALSE;
	return(new);	
}

char *nfgetcs(NEWFILE *file, char testchar)
{
	char *bufferstart = file->buffer;
	int bufferlen = file->bufferlen;
	char *bptr, *result;
	int slen;
	
	if (file->eof)
		return(NULL);

	while((bptr = memchr(bufferstart, (int)testchar, bufferlen)) == NULL)
	{

		slen = bufferlen;
		bufferlen = readbuffer(file, 1024); 
		bufferstart = file->buffer + slen;
		
		if (bufferlen == 0)
		{
			bptr = bufferstart - 1;
			break;
		} 
		else if (bufferlen == -1)
			return(NULL);
	}
	
	/* set up the result */
	slen = bptr - file->buffer + 1;	
	
	result = mallocwrapper(slen + 1);
	memcpy(result, file->buffer, slen);
	result[slen] = 0;
	
	/* fix up the buffer */
	
	file->bufferlen -= slen; 
	memmove(file->buffer, bptr+1, file->bufferlen);
	
	return(result);
}

void nfclose(NEWFILE *file)
{
	close(file->fd);
	freewrapper(file->buffer);
	freewrapper(file);
}
