/* Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "ftpd.h"

void pnums_init(void)
{
	int count;
#if DEBUG
	printf("[init_pnums]\n");
#endif
	pnums = (THREADSTAT *)shmem_malloc(sizeof(THREADSTAT) * (config->maxthreads + 1));
 	for(count = 0; count <= config->maxthreads; count++)
 	{
 		pnums[count].pid = (pid_t)0;
 		pnums[count].remoteip = 0;
		strclear(pnums[count].username);
		strclear(pnums[count].currentop);
 	}
} 

/* pid is only be examined or set by the parent. so i don't need to lock */

int pnums_getnum_start(void)
{
	int count = 0;
	
	while(((int)pnums[count].pid != 0) && (count < config->maxthreads))
		count++;
	
	if (count == config->maxthreads)
		count = -1;
	
	return(count);
}

void pnums_getnum_finish(int processnum, pid_t pid, int ipaddress)
{
	shmem_lock(processnum + 1);
	pnums[processnum].pid = pid;
	pnums[processnum].remoteip = ipaddress;
	strcpy(pnums[processnum].username, "<unknown>");
	shmem_unlock(processnum + 1);
}

/* not safe to lock :( */

void pnums_free_bypid(pid_t pid)
{
	int count = 0;
	
	/* find the pid */
	while((pnums[count].pid != pid) && (count < config->maxthreads))
		count++;
	
	if (count != config->maxthreads)
	{
		pnums[count].pid = (pid_t)0;
		pnums[count].remoteip = 0;
	}
}

void pnums_setusername(int threadnum, char *username)
{
	shmem_lock(threadnum + 1);
	strncpy(pnums[threadnum].username, username, MAXUSERLEN);
	shmem_unlock(threadnum + 1);
}

/* I must do something to this at another time, too slow for LOTS of users! */

int pnums_numusers(void)
{
	int count;
	int result = 0;
	
	for (count = 0; count < config->maxthreads; count++)
	{
		shmem_lock(count + 1);
		if ((int)pnums[count].pid > 0)
			result++;
		shmem_unlock(count + 1);
	}
	
	return(result);
}

int pnums_countuser(char *username)
{
	int count;
	int result = 0;
	
	for (count = 0; count < config->maxthreads; count++)
	{
		shmem_lock(count + 1);
		if ((int)pnums[count].pid > 0)
		{
			if (strcmp(username, pnums[count].username) == 0)
				result++;
		}
		shmem_unlock(count + 1);
	}
	
	return(result);
}

void pnums_killchildren(void)
{
#if DEBUG
	printf("[pnums_killchildren]\n");
#endif
	if (pnums != NULL)
	{
		int count;
		for (count = 0; count < config->maxthreads; count++)
			if ((int)pnums[count].pid > 0)
				kill(pnums[count].pid, SIGTERM);
	}
}

void pnums_setop(char *newop, int num)
{
	shmem_lock(num + 1);
	strncpy(pnums[num].currentop, newop, DESCRIPLEN);
	shmem_unlock(num + 1);
}
	
void pnums_shutdown(void)
{
#if DEBUG
	printf("[pnums_shutdown]\n");
#endif
	if (pnums != NULL)
	{
		int count;
		
		for (count = 0; count < config->maxthreads; count++)
			if (pnums[count].username)
				 
		shmem_free(pnums);
		pnums = NULL;
	}
}
