/* Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "ftpd.h"
#include "ftpcmd.h"

extern FTPCMD mainftpcmd[];

int controlportgotdata(SELECTER *sel, void *peerdata)
{
	FTPSTATE *peer = (FTPSTATE *)peerdata;
	char *instring = NULL;
	int connclosed = readcmd(peer);
	int exits = FALSE;
	INPUTLINE cmd;

	if (connclosed)
		return(TRUE);
	else 
	{
		while ((!exits) && popcmd(peer, &instring))
		{
			cmd_split(peer, &cmd, instring, mainftpcmd, TRUE);
			if (!peer->dport)
				if (cmd.command->ftpfunc != ftp_pass)
					pnums_setop(instring, peer->threadnum);
				
			exits = ftp_run(peer, &cmd, "FTP"); 
			if (cmd.parameters != NULL)
				freewrapper(cmd.parameters);
 			if (!peer->dport)
				pnums_setop("idle", peer->threadnum);
				 			
 			freewrapper(instring);
 		}	
		return(exits);
	}
}

void ftpserverside_main(int remotefd, int remoteip, int threadnum, int portnum)
{
	int exits = FALSE;
	FTPSTATE *peer = mallocwrapper(sizeof(FTPSTATE));
	NEWFILE *logdump = nfopen(config->prelogondump);
	
	signal(SIGHUP, SIG_IGN);

	ftpstate_init(peer, remotefd, remoteip, threadnum, portnum); 
		/* initalize ftp state data struture */
	peer->sel = select_new();
	
	/* disable nagle alogrithm for performace */
	nagleoff(remotefd);
			
#ifdef DEBUG
	printf("[ftpserverside_main socket: %d]\n", remotefd);
#endif	
	/* add the remote fd for reading */
	select_addfd(peer->sel, remotefd);
	select_addread(peer->sel, remotefd, controlportgotdata, (void *)peer);
	
	/* output our greeting */
	pnums_setop("idle", peer->threadnum);
	exits = ftp_dumper(peer, logdump, 220, PROGNAME" ("VERSTR") server ready. Enter Username.", TRUE);

 	while(!exits)
 	{
		int fd, signalnum;
		
 		fd = select_do(peer->sel, &signalnum, peer->timeout);
		/* if we get 3 back, simply loop and update timeout */
 		if (fd == -1)
 		{
			if (signalnum == SIGTERM)
 			{
 				ftp_write(peer, 421, "Goodbye, server shutting down, logging out.");
 				log_addentry(MYLOG_LOGIN, peer->remoteip, peer->username, "Logging out (server shutdown)"); 
 				exits = TRUE;
 			}
 			else if (signalnum == 0)
 				exits = TRUE;
		
		}
		else if (fd == 0)
 		{
 			ftp_write(peer, 421, "Goodbye, timeout occured (%d sec), logging out.", peer->timeout);
			log_addentry(MYLOG_LOGIN, peer->remoteip, peer->username, "Logging out (timeout)");
 			exits = TRUE;
 		} 
 		else if (fd != 3)
 			exits = TRUE;
 	}
 		
	acl_clearcache(peer);
	select_shutdown(peer->sel);
	ftpstate_dest(peer);
}

int inport_bind(int portid, unsigned int bindip)
{
	int incon;
	
#ifdef DEBUG
	printf("[inport_bind port: %d]\n", portid);
#endif
	if ((portid < 1024) && (getuid() != 0))
	{
		char *dat = safe_snprintf("root access required to bind port %d (less than 1024)", portid);
		ERRORMSGFATAL(dat);
	}
	
	incon = listenport(portid, bindip, MAXPENDCONN);
	if (incon == -1)
	{
		char *dat = safe_snprintf("Cannot bind to port %d!", portid);
		ERRORMSGFATAL(dat);
	}
	if (incon == -2)
		ERRORMSGFATAL("Serious internal error binding to port."); 

	return(incon);
}

int inport_getconn(SELECTER *mainsel, void *vincon)
{
	int newconn;
	unsigned int ipaddress;
	int *incon = vincon;
	
#ifdef DEBUG
	printf("[inport_getconn port: %d]\n", *incon);
#endif		 

	newconn = get_conn(*incon, &ipaddress); 
	if (!user_allowed(ipaddress))
	{
		log_addentry(MYLOG_DACCESS, ipaddress, NULL, NULL);
		close(newconn);
	}
	else
	{
		int tnum, port;
		pid_t fresult;
		
		tnum = pnums_getnum_start();
		port = *((incon - config->listenfds) + config->listenport);
		
		if (tnum != -1)
		{
			fresult = forkwrapper();
			if ((int)fresult == 0)
			{
				select_shutdown(mainsel);
				ftpserverside_main(newconn, ipaddress, tnum, port);
				ftpd_shutdown();
				exit(0);
			}
			else
			{
				close(newconn);
				pnums_getnum_finish(tnum, fresult, ipaddress);
			}
		}
		else 
		{
			errormsg("Maximum users reached!, increase maxthreads", __FILE__, __LINE__);
			log_addentry(MYLOG_INFO, 0, NULL, "Reached maxthreads limit. Server too busy.");
			close(newconn);
		}
	}
	return(FALSE);
}
