/* Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "ftpd.h"
#include "shmem.h"

/* this is a simple allocation routine. You must allocate and unallocate
   like a stack */
   
void *shmem_malloc(int size)
{
	void *new;
	
	assert(size > 0);
	
	shmem_lock(0);		/* get the main lock, I am paranoid */

	if (shmemdata->size < size)
		ERRORMSGFATAL("Out of shared memory!");
	shmemdata->size -= size; /* allocate the space */
	new = shmemdata->top;
	shmemdata->top = (char *)shmemdata->top + size;  /* increament the pointer */
	
	shmem_unlock(0);
	return(new);
}	

void shmem_free(void *newpos)
{
	shmem_lock(0);

	assert(newpos > shmemdata->bottom);
	assert(newpos < shmemdata->top);
	
	shmemdata->size += ((char *)shmemdata->top - (char *)newpos);
	shmemdata->top = newpos;
	
	shmem_unlock(0);
}

void shmem_finish()
{
	struct shmid_ds buffer;
	if (shmemdata)
	{
		if (shmctl(shmemdata->shmemnum, IPC_RMID, &buffer) == -1);
		close(shmem_lockfd);
	}
}

void shmem_init(char *configfile)
{
	int shmemnum;
	void *shmemptr;
	key_t ipckey = ftok(configfile, '/');
	
#ifdef DEBUG
	printf("[shmat_init with key %d]\n", (int)ipckey);
#endif	
	shmemnum = shmget(ipckey, 1024 * SHMEMTOTAL, 
					IPC_CREAT | /*IPC_EXCL |*/ 0600);
	if (shmemnum == -1)
		ERRORMSGFATAL(strerror(errno));
	
	shmemptr = shmat(shmemnum, NULL, 0);
	shmemdata = shmemptr;
	shmemdata->shmemnum = shmemnum;
	shmemdata->size = (1024 * SHMEMTOTAL) - sizeof(struct shmemctr);
	shmemdata->bottom = shmemptr;
	shmemdata->top = (char *)shmemptr + sizeof(struct shmemctr);
	shmemdata->pnum_count = config->maxthreads;
	
	/* get semaphores. Using the config file so others can join in (we use file locks for portability, simplicity
	   and usability. */
	
	shmem_lockfd = open(configfile, O_RDWR);
	if (shmem_lockfd == -1)
		ERRORMSGFATAL(strerror(errno));
}
