/* Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#define SHMEMTOTAL	32	/* We get this much shared memory (K) */

int shmem_lockfd;

struct shmemctr
{
	void *bottom;
	void *top;
	int shmemnum;
	int size;
	int pnum_count;		/* to be removed later :( */
} *shmemdata = NULL;

/* lock the shared memory region */

void shmem_lock(int id)
{
	struct flock lock;
	register int err;
	
	assert(id >= 0);
	
	lock.l_type = F_WRLCK;
	lock.l_whence = SEEK_SET;
	lock.l_start = id + 1;
	lock.l_len = 1;
	
	NOSIGNALINTR(err = fcntl(shmem_lockfd, F_SETLKW, &lock));
	if (err == -1)
		ERRORMSGFATAL(strerror(errno));
}

/* unlock the shared memory region */

void shmem_unlock(int id)
{
	struct flock lock;
	register int err;
	
	assert(id >= 0);

	lock.l_type = F_UNLCK;
	lock.l_whence = SEEK_SET;
	lock.l_start = id + 1;
	lock.l_len = 1;

	/* XXXX must check for EINTR */
	NOSIGNALINTR(err = fcntl(shmem_lockfd, F_SETLKW, &lock));
	if (err == -1)
		ERRORMSGFATAL(strerror(errno));
}

void shmem_stats(void)
{
	shmem_lock(0);
	printf("SHARED MEMORY STATS\n");
	printf("==========================================================\n");
	printf("Total shared memory:    %d\n", 1024 * SHMEMTOTAL);
	printf("Total free memory:      %d\n", shmemdata->size);
	printf("Total used memory:	%d\n", 1024 * SHMEMTOTAL - shmemdata->size);
	printf("==========================================================\n");
	shmem_unlock(0);
}
