/* Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "ftpd.h"

int strchrcount(char *string, char tok)
{
	char *lastoccur = string;
	int ret = 0;
	
	while((lastoccur = strchr(lastoccur, tok)) != NULL)
	{
		lastoccur++;
		ret++;
	}
	return(ret);
}

/* simple function to clear a string. */

void strclear(char *string)
{
	assert(string != NULL);

	string[0] = 0;		/* clear the string quickly */
}

/* another function that will do a strncpy if it doesn't clobber the 
   destination string */

char *strdupifnull(char *in, char *data)
{
	if (in == NULL)
		return(strdupwrapper(data));
	else
		return(in);
}

/* This has been simplified to speed up code */

char *safe_snprintf(char *format, ...)
{
	int size = BUFFERSIZE;
	va_list printfargs;
	char *buffer = mallocwrapper(size);
	int result;
	
	va_start(printfargs, format);
	result = vsnprintf(buffer, size, format, printfargs);
	va_end(printfargs);
	
	if (result == -1)
	{
		size = MAXMSGLEN;
		reallocwrapper(size, (void *)&buffer);
		
		va_start(printfargs, format);
		result = vsnprintf(buffer, size, format, printfargs);
		va_end(printfargs);
	}
	
	if (result == -1)
		/* stuff is wrong, bail out! */
		ERRORMSGFATAL("String too long!");
	
	return(buffer);
}


char *gethostname2(void)
{
	NEWFILE *hostnamefile = nfopen("/etc/HOSTNAME");
	char *output;
	
	if (hostnamefile == NULL)
		return(strdupwrapper("<unknown>"));
	
	if ((output = nfgetcs(hostnamefile, '\n')) != NULL)
	{
		if (output[strlen(output)-1] == '\n')
			output[strlen(output)-1] = 0; 
		nfclose(hostnamefile);
		return(output);
	}
	nfclose(hostnamefile);
	return(strdupwrapper("<unknown>"));
}
