/* Copyright (C) 1999 Beau Kuiper

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "ftpd.h"

TOKENSET *tokenset_new()
{
	TOKENSET *newset = mallocwrapper(sizeof (TOKENSET));

	memset(newset, 0, sizeof(TOKENSET));
	return(newset);
}

void tokenset_settoken(TOKENSET *tset, unsigned char tok, char *data)
{
	int token = (int)tok;
	
	assert(token < 128);
	assert(tset != NULL);

/*	printf("adding a token %d %s\n", token, data); */
		
	if(tset->params[token] != NULL)
		freewrapper(tset->params[token]);
	
	tset->params[token] = data;
}

void tokenset_deltoken(TOKENSET *tset, unsigned char tok)
{
	int token = (int)tok;

	assert(token < 128);
	assert(tset != NULL);

	if (tset->params[token] != NULL)
		freewrapper(tset->params[token]);
	
	tset->params[token] = NULL;
}

char *tokenset_apply(TOKENSET *tok, char *inputstr)
{
	int instrlen;
	char *outstring;
	int inpos = 0;
	int outpos = 0;
	int outstrlen;
	
	if (inputstr == NULL)
		return(NULL);
		
	instrlen = strlen(inputstr);
	outstring = mallocwrapper(instrlen+1);	
	outstrlen = instrlen + 1;
	
	while(inpos < instrlen)
	{
		/* WARNING: This code contains strange things. Look carefully
			before frobing */
			
		if (inputstr[inpos] != '%')
			outstring[outpos++] = inputstr[inpos++];
		else if (inputstr[inpos + 1] == '%')
		{
			outstring[outpos++] = '%';
			inpos += 2;
		}
		else if (inputstr[inpos + 1] == '(')
		{
			int a, b, strl, pc;
			
			if (sscanf(inputstr + inpos, "%%(%d,%d)", &a, &b) < 2)
				goto fail;
			while(inputstr[inpos++] != ')')
				if (inputstr[inpos] == 0)
					goto fail;
			pc = (int)inputstr[inpos];
			if (pc >= 128)
				goto fail;
			if (tok->params[pc] == NULL)
				goto fail;
			if (a < 0)
				a = 0;
			if (a > b)
				goto fail;
			strl = b - a + 1;
			if (strl > strlen(tok->params[pc]) - a)
				strl = strlen(tok->params[pc]) - a;
			inpos++;
			
			outstrlen += strl;
			reallocwrapper(outstrlen+1, (void *)&outstring);
			 
			strncpy(outstring + outpos, tok->params[pc] + a, strl);
			outpos += strl; 
		}
		else
		{
			int strl, pc;
			
			pc = (int)inputstr[++inpos];
			if (pc >= 128)
				goto fail;
			if (tok->params[pc] == NULL)
				goto fail;
			strl = strlen(tok->params[pc]);

			outstrlen += strl;
			reallocwrapper(outstrlen+1, (void *)&outstring);
			strncpy(outstring + outpos, tok->params[pc], strl);
			inpos++;
			outpos += strl;
		}
	}
	freewrapper(inputstr);
	outstring[outpos] = 0;
	return(outstring);

fail:
	freewrapper(outstring);

	/* log the error so it can get fixed, because it may cause security
	   problems when cookies don't parse correctly! */
	   
	outstring = safe_snprintf("Failed to parse cookie in line: '%s'", inputstr);
	log_addentry(MYLOG_INFO, 0, NULL, outstring);
	freewrapper(outstring);
	return(inputstr);
}

void tokenset_finish(TOKENSET *tset)
{
	int count;
	
	for (count = 0; count < 128; count++)
	{
		if (tset->params[count])
			freewrapper(tset->params[count]);
	}
	freewrapper(tset);
}
