/* Copyright (C) 1999 Beau Kuiper.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this program; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include "../ftpd.h"

#ifndef HAVE_GLOB

#include <dirent.h>

char *wildcardtoregexp(char *wildcardstr)
{
	char *output = mallocwrapper(strlen(wildcardstr) * 2 + 10);
	int pos = 0;
	int pos2 = 1;
	output[0] = '^';

	while (wildcardstr[pos] != 0)
	{
		switch(wildcardstr[pos])
		{	
			case '*':
				output[pos2] = '.'; pos2++;
				output[pos2] = '*'; pos2++;
				break;
			case '?':
				output[pos2] = '.'; pos2++;
				break;
			case '+':
			case '^':
			case '$':
			case '.':
				output[pos2] = '\\'; pos2++;
				output[pos2] = wildcardstr[pos]; pos2++;
				break;
			default:
				output[pos2] = wildcardstr[pos]; pos2++;
				break;
		}
		pos++;
	}
	
	output[pos2] = '$'; output[pos2 + 1] = '\0';
	return(output);
}

int glob (char *pattern, int flags,
		      int (*errfunc) (const char *, int),
		      glob_t *pglob)
{
	char *dir = strdupwrapper(pattern);
	char *wildpattern = strrchr(dir, '/');
	char *regpattern = NULL;
	int error = 0;
	DIR *globdir = NULL;
	int grex = 0;
	
	*wildpattern = 0; /* split dir into the dir part and the pattern part */
	wildpattern++;
	
	pglob->gl_pathc = 0;
	pglob->gl_pathv = NULL; /* Make sure globfree works ok */
	
	error = ((globdir = opendir(dir)) == NULL);
	
	if (!error)
	{
		regpattern = wildcardtoregexp(wildpattern); 
		error = (re_comp(regpattern) != NULL);
	}
		
	if (!error)
	{
		struct dirent *filedat;
		
		grex = TRUE;
		while((filedat = readdir(globdir)) != NULL)
		{
			if (re_exec(filedat->d_name) == 1)
			{
				(pglob->gl_pathc)++;
				reallocwrapper(sizeof(char **) * pglob->gl_pathc, (void *)&(pglob->gl_pathv));
				pglob->gl_pathv[pglob->gl_pathc - 1] = safe_snprintf("%s/%s", dir, filedat->d_name);
			}
		}
	}
	
	reallocwrapper(sizeof(char **) * (pglob->gl_pathc + 1), (void *)&(pglob->gl_pathv));
	pglob->gl_pathv[pglob->gl_pathc] = NULL;
	 
	if (globdir)
		closedir(globdir);
	
	if (regpattern)
		freewrapper(regpattern);
		
	freewrapper(dir);
	return(error);
}	

void globfree (glob_t *pglob)
{
	int count;
	if (pglob->gl_pathv)
	{ 
		for (count = 0; count < pglob->gl_pathc; count++)
			freewrapper(pglob->gl_pathv[count]);
			
		freewrapper(pglob->gl_pathv);
	}
}

#endif /* HAVE_GLOB */
